@@ -2,7 +2,24 @@
  * @file methods/ann/activation_functions/hyper_sinh_function.hpp
  * @author Mayank Raj
  *
- * Definition and implementation of the Hyper-sinh function.
+ * Definition of the hyper-sinh function as described by
+ * L. Parisi et al. (2021) (full reference below).
+ *
+ * Original implementation in Python by Luca Parisi as per https://github.com/luca-parisi/hyper_sinh
+ *
+ * For more information, see the following paper.
+ *
+ * @code
+ * @article{parisi2021hyper,
+ *   title={hyper-sinh: An accurate and reliable function from shallow to deep learning in TensorFlow and Keras},
+ *   author={Parisi, Luca and Ma, Renfei and RaviChandran, Narrendar and Lanzillotta, Matteo},
+ *   journal={Machine Learning with Applications},
+ *   volume={6},
+ *   pages={100112},
+ *   year={2021},
+ *   publisher={Elsevier}
+ * }
+ * @endcode
  *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
@@ -77,7 +94,7 @@ class HyperSinhFunction
    * @param y Input activation.
    * @return f'(x)
    */
-  static double Deriv(const double y)
+  static double Deriv(const double /* x */, const double y)
   {
     if (y > 0)
       return (std::pow((1.0 / 9.0) + (y * y), 0.5));
@@ -89,22 +106,22 @@ class HyperSinhFunction
    * Computes the first derivatives of the Hyper-sinh function.
    *
    * @param y Input activations.
-   * @param x The resulting derivatives.
+   * @param dy The resulting derivatives.
    */
-  template<typename InputVecType, typename OutputVecType>
-  static void Deriv(const InputVecType& y, OutputVecType& x)
+  template<typename InputVecType, typename OutputVecType, typename DerivVecType>
+  static void Deriv(const InputVecType& /* x */, const OutputVecType& y, DerivVecType& dy)
   {
-    x.set_size(y.size());
+    dy.set_size(size(y));
     #pragma omp for
     for(size_t i = 0; i < y.n_elem; ++i)
     {
       if (y(i) > 0)
       {
-        x(i) = (std::pow((1.0 / 9.0) + (y(i) * y(i)), 0.5));
+        dy(i) = (std::pow((1.0 / 9.0) + (y(i) * y(i)), 0.5));
       }
       else
       {
-        x(i) = (3.0 * std::pow(std::pow(y(i), 2) / 4, 1.0 / 3.0));
+        dy(i) = (3.0 * std::pow(std::pow(y(i), 2) / 4, 1.0 / 3.0));
       }
     }
   }
