@@ -1,5 +1,5 @@
 /**
- * @file cover_tree_impl.hpp
+ * @file core/tree/cover_tree/cover_tree_impl.hpp
  * @author Ryan Curtin
  *
  * Implementation of CoverTree class.
@@ -15,10 +15,22 @@
 // In case it hasn't already been included.
 #include "cover_tree.hpp"
 
+#include <queue>
 #include <string>
 
 namespace mlpack {
-namespace tree {
+
+// Build the statistics, bottom-up.
+template<typename TreeType, typename StatisticType>
+void BuildStatistics(TreeType* node)
+{
+  // Recurse first.
+  for (size_t i = 0; i < node->NumChildren(); ++i)
+    BuildStatistics<TreeType, StatisticType>(&node->Child(i));
+
+  // Now build the statistic.
+  node->Stat() = StatisticType(*node);
+}
 
 // Create the cover tree.
 template<
@@ -87,9 +99,8 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
     {
       children.push_back(&(old->Child(i)));
 
-      // Set its parent correctly, and rebuild the statistic.
+      // Set its parent correctly.
       old->Child(i).Parent() = this;
-      old->Child(i).Stat() = StatisticType(old->Child(i));
     }
 
     // Remove all the children so they don't get erased.
@@ -103,14 +114,20 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
   }
 
   // Use the furthest descendant distance to determine the scale of the root
-  // node.
-  if (furthestDescendantDistance == 0.0)
+  // node.  Note that if the root is a leaf, we can have scale INT_MIN, but if
+  // it *isn't* a leaf, we need to mark the scale as one higher than INT_MIN, so
+  // that the recursions don't fail.
+  if (furthestDescendantDistance == 0.0 && dataset.n_cols == 1)
     scale = INT_MIN;
+  else if (furthestDescendantDistance == 0.0)
+    scale = INT_MIN + 1;
   else
-    scale = (int) ceil(log(furthestDescendantDistance) / log(base));
+    scale = (int) std::ceil(std::log(furthestDescendantDistance) /
+        std::log(base));
 
-  // Initialize statistic.
-  stat = StatisticType(*this);
+  // Initialize statistics recursively after the entire tree construction is
+  // complete.
+  BuildStatistics<CoverTree, StatisticType>(this);
 
   Log::Info << distanceComps << " distance computations during tree "
       << "construction." << std::endl;
@@ -134,9 +151,9 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
     parent(NULL),
     parentDistance(0),
     furthestDescendantDistance(0),
-    localMetric(false),
+    localMetric(true),
     localDataset(false),
-    metric(&metric),
+    metric(new MetricType(metric)),
     distanceComps(0)
 {
   // If there is only one point or zero points in the dataset... uh, we're done.
@@ -180,8 +197,6 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
 
       // Set its parent correctly.
       old->Child(i).Parent() = this;
-      // Rebuild the statistic.
-      old->Child(i).Stat() = StatisticType(old->Child(i));
     }
 
     // Remove all the children so they don't get erased.
@@ -195,14 +210,20 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
   }
 
   // Use the furthest descendant distance to determine the scale of the root
-  // node.
-  if (furthestDescendantDistance == 0.0)
+  // node.  Note that if the root is a leaf, we can have scale INT_MIN, but if
+  // it *isn't* a leaf, we need to mark the scale as one higher than INT_MIN, so
+  // that the recursions don't fail.
+  if (furthestDescendantDistance == 0.0 && dataset.n_cols == 1)
     scale = INT_MIN;
+  else if (furthestDescendantDistance == 0.0)
+    scale = INT_MIN + 1;
   else
-    scale = (int) ceil(log(furthestDescendantDistance) / log(base));
+    scale = (int) std::ceil(std::log(furthestDescendantDistance) /
+        std::log(base));
 
-  // Initialize statistic.
-  stat = StatisticType(*this);
+  // Initialize statistics recursively after the entire tree construction is
+  // complete.
+  BuildStatistics<CoverTree, StatisticType>(this);
 
   Log::Info << distanceComps << " distance computations during tree "
       << "construction." << std::endl;
@@ -271,9 +292,8 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
     {
       children.push_back(&(old->Child(i)));
 
-      // Set its parent correctly, and rebuild the statistic.
+      // Set its parent correctly.
       old->Child(i).Parent() = this;
-      old->Child(i).Stat() = StatisticType(old->Child(i));
     }
 
     // Remove all the children so they don't get erased.
@@ -287,14 +307,20 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
   }
 
   // Use the furthest descendant distance to determine the scale of the root
-  // node.
-  if (furthestDescendantDistance == 0.0)
+  // node.  Note that if the root is a leaf, we can have scale INT_MIN, but if
+  // it *isn't* a leaf, we need to mark the scale as one higher than INT_MIN, so
+  // that the recursions don't fail.
+  if (furthestDescendantDistance == 0.0 && dataset->n_cols == 1)
     scale = INT_MIN;
+  else if (furthestDescendantDistance == 0.0)
+    scale = INT_MIN + 1;
   else
-    scale = (int) ceil(log(furthestDescendantDistance) / log(base));
+    scale = (int) std::ceil(std::log(furthestDescendantDistance) /
+        std::log(base));
 
-  // Initialize statistic.
-  stat = StatisticType(*this);
+  // Initialize statistics recursively after the entire tree construction is
+  // complete.
+  BuildStatistics<CoverTree, StatisticType>(this);
 
   Log::Info << distanceComps << " distance computations during tree "
       << "construction." << std::endl;
@@ -318,9 +344,9 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
     parent(NULL),
     parentDistance(0),
     furthestDescendantDistance(0),
-    localMetric(false),
+    localMetric(true),
     localDataset(true),
-    metric(&metric),
+    metric(new MetricType(metric)),
     distanceComps(0)
 {
   // If there is only one point or zero points in the dataset... uh, we're done.
@@ -362,9 +388,8 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
     {
       children.push_back(&(old->Child(i)));
 
-      // Set its parent correctly, and rebuild the statistic.
+      // Set its parent correctly.
       old->Child(i).Parent() = this;
-      old->Child(i).Stat() = StatisticType(old->Child(i));
     }
 
     // Remove all the children so they don't get erased.
@@ -378,14 +403,20 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
   }
 
   // Use the furthest descendant distance to determine the scale of the root
-  // node.
-  if (furthestDescendantDistance == 0.0)
+  // node.  Note that if the root is a leaf, we can have scale INT_MIN, but if
+  // it *isn't* a leaf, we need to mark the scale as one higher than INT_MIN, so
+  // that the recursions don't fail.
+  if (furthestDescendantDistance == 0.0 && dataset->n_cols == 1)
     scale = INT_MIN;
+  else if (furthestDescendantDistance == 0.0)
+    scale = INT_MIN + 1;
   else
-    scale = (int) ceil(log(furthestDescendantDistance) / log(base));
+    scale = (int) std::ceil(std::log(furthestDescendantDistance) /
+        std::log(base));
 
-  // Initialize statistic.
-  stat = StatisticType(*this);
+  // Initialize statistics recursively after the entire tree construction is
+  // complete.
+  BuildStatistics<CoverTree, StatisticType>(this);
 
   Log::Info << distanceComps << " distance computations during tree "
       << "construction." << std::endl;
@@ -428,15 +459,11 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
   {
     this->scale = INT_MIN;
     numDescendants = 1;
-    stat = StatisticType(*this);
     return;
   }
 
   // Otherwise, create the children.
   CreateChildren(indices, distances, nearSetSize, farSetSize, usedSetSize);
-
-  // Initialize statistic.
-  stat = StatisticType(*this);
 }
 
 // Manually create a cover tree node.
@@ -471,11 +498,9 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
   // If necessary, create a local metric.
   if (localMetric)
     this->metric = new MetricType();
-
-  // Initialize the statistic.
-  stat = StatisticType(*this);
 }
 
+// Copy Constructor.
 template<
     typename MetricType,
     typename StatisticType,
@@ -494,9 +519,9 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
     parent(other.parent),
     parentDistance(other.parentDistance),
     furthestDescendantDistance(other.furthestDescendantDistance),
-    localMetric(false),
+    localMetric(other.localMetric),
     localDataset(other.parent == NULL && other.localDataset),
-    metric(other.metric),
+    metric((other.localMetric ? new MetricType() : other.metric)),
     distanceComps(0)
 {
   // Copy each child by hand.
@@ -526,6 +551,76 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
   }
 }
 
+// Copy assignment operator: copy the given other tree.
+template<
+    typename MetricType,
+    typename StatisticType,
+    typename MatType,
+    typename RootPointPolicy
+>
+CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>&
+CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
+operator=(const CoverTree& other)
+{
+  if (this == &other)
+    return *this;
+
+  // Freeing memory that will not be used anymore.
+  if (localDataset)
+    delete dataset;
+
+  if (localMetric)
+    delete metric;
+
+  for (size_t i = 0; i < children.size(); ++i)
+    delete children[i];
+  children.clear();
+
+  dataset = ((other.parent == NULL && other.localDataset) ?
+      new MatType(*other.dataset) : other.dataset);
+  point = other.point;
+  scale = other.scale;
+  base = other.base;
+  stat = other.stat;
+  numDescendants = other.numDescendants;
+  parent = other.parent;
+  parentDistance = other.parentDistance;
+  furthestDescendantDistance = other.furthestDescendantDistance;
+  localMetric = other.localMetric;
+  localDataset = (other.parent == NULL && other.localDataset);
+  metric = (other.localMetric ? new MetricType() : other.metric);
+  distanceComps = 0;
+
+  // Copy each child by hand.
+  for (size_t i = 0; i < other.NumChildren(); ++i)
+  {
+    children.push_back(new CoverTree(other.Child(i)));
+    children[i]->Parent() = this;
+  }
+
+  // Propagate matrix, but only if we are the root.
+  if (parent == NULL && localDataset)
+  {
+    std::queue<CoverTree*> queue;
+
+    for (size_t i = 0; i < NumChildren(); ++i)
+      queue.push(children[i]);
+
+    while (!queue.empty())
+    {
+      CoverTree* node = queue.front();
+      queue.pop();
+
+      node->dataset = dataset;
+      for (size_t i = 0; i < node->NumChildren(); ++i)
+        queue.push(node->children[i]);
+    }
+  }
+
+  return *this;
+}
+
+// Move Constructor.
 template<
     typename MetricType,
     typename StatisticType,
@@ -566,7 +661,65 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
   other.metric = NULL;
 }
 
-// Construct from a boost::serialization archive.
+// Move assignment operator: take ownership of the given tree.
+template<
+    typename MetricType,
+    typename StatisticType,
+    typename MatType,
+    typename RootPointPolicy
+>
+CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>&
+CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
+operator=(CoverTree&& other)
+{
+  if (this == &other)
+    return *this;
+
+  // Freeing memory that will not be used anymore.
+  if (localDataset)
+    delete dataset;
+
+  if (localMetric)
+    delete metric;
+
+  for (size_t i = 0; i < children.size(); ++i)
+    delete children[i];
+
+  dataset = other.dataset;
+  point = other.point;
+  children = std::move(other.children);
+  scale = other.scale;
+  base = other.base;
+  stat = std::move(other.stat);
+  numDescendants = other.numDescendants;
+  parent = other.parent;
+  parentDistance = other.parentDistance;
+  furthestDescendantDistance = other.furthestDescendantDistance;
+  localMetric = other.localMetric;
+  localDataset = other.localDataset;
+  metric = other.metric;
+  distanceComps = other.distanceComps;
+
+  // Set proper parent pointer.
+  for (size_t i = 0; i < children.size(); ++i)
+    children[i]->Parent() = this;
+
+  other.dataset = NULL;
+  other.point = 0;
+  other.scale = INT_MIN;
+  other.base = 0;
+  other.numDescendants = 0;
+  other.parent = NULL;
+  other.parentDistance = 0;
+  other.furthestDescendantDistance = 0;
+  other.localMetric = false;
+  other.localDataset = false;
+  other.metric = NULL;
+
+  return *this;
+}
+
+// Construct from a cereal archive.
 template<
     typename MetricType,
     typename StatisticType,
@@ -576,11 +729,11 @@ template<
 template<typename Archive>
 CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree(
     Archive& ar,
-    const typename boost::enable_if<typename Archive::is_loading>::type*) :
+    const typename std::enable_if_t<cereal::is_loading<Archive>()>*) :
     CoverTree() // Create an empty CoverTree.
 {
   // Now, serialize to our empty tree.
-  ar >> data::CreateNVP(*this, "tree");
+  ar(cereal::make_nvp("this", *this));
 }
 
 
@@ -662,7 +815,7 @@ template<typename MetricType,
 template<typename VecType>
 size_t CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
     GetNearestChild(const VecType& point,
-                    typename boost::enable_if<IsVector<VecType> >::type*)
+                    typename std::enable_if_t<IsVector<VecType>::value>*)
 {
   if (IsLeaf())
     return 0;
@@ -692,7 +845,7 @@ template<typename MetricType,
 template<typename VecType>
 size_t CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
     GetFurthestChild(const VecType& point,
-                     typename boost::enable_if<IsVector<VecType> >::type*)
+                     typename std::enable_if_t<IsVector<VecType>::value>*)
 {
   if (IsLeaf())
     return 0;
@@ -897,7 +1050,7 @@ template<
     typename MatType,
     typename RootPointPolicy
 >
-math::RangeType<typename
+RangeType<typename
     CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::ElemType>
 CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
     RangeDistance(const CoverTree& other) const
@@ -905,9 +1058,9 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
   const ElemType distance = metric->Evaluate(dataset->col(point),
       other.Dataset().col(other.Point()));
 
-  math::RangeType<ElemType> result;
-  result.Lo() = distance - furthestDescendantDistance -
-      other.FurthestDescendantDistance();
+  RangeType<ElemType> result;
+  result.Lo() = std::max(distance - furthestDescendantDistance -
+      other.FurthestDescendantDistance(), 0.0);
   result.Hi() = distance + furthestDescendantDistance +
       other.FurthestDescendantDistance();
 
@@ -922,15 +1075,15 @@ template<
     typename MatType,
     typename RootPointPolicy
 >
-math::RangeType<typename
+RangeType<typename
     CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::ElemType>
 CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
     RangeDistance(const CoverTree& other,
                   const ElemType distance) const
 {
-  math::RangeType<ElemType> result;
-  result.Lo() = distance - furthestDescendantDistance -
-      other.FurthestDescendantDistance();
+  RangeType<ElemType> result;
+  result.Lo() = std::max(distance - furthestDescendantDistance -
+      other.FurthestDescendantDistance(), 0.0);
   result.Hi() = distance + furthestDescendantDistance +
       other.FurthestDescendantDistance();
 
@@ -944,15 +1097,16 @@ template<
     typename MatType,
     typename RootPointPolicy
 >
-math::RangeType<typename
+RangeType<typename
     CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::ElemType>
 CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
     RangeDistance(const arma::vec& other) const
 {
   const ElemType distance = metric->Evaluate(dataset->col(point), other);
 
-  return math::RangeType<ElemType>(distance - furthestDescendantDistance,
-                     distance + furthestDescendantDistance);
+  return RangeType<ElemType>(
+      std::max(distance - furthestDescendantDistance, 0.0),
+      distance + furthestDescendantDistance);
 }
 
 //! Return the minimum and maximum distance to another point given that the
@@ -963,14 +1117,15 @@ template<
     typename MatType,
     typename RootPointPolicy
 >
-math::RangeType<typename
+RangeType<typename
     CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::ElemType>
 CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
     RangeDistance(const arma::vec& /* other */,
                   const ElemType distance) const
 {
-  return math::RangeType<ElemType>(distance - furthestDescendantDistance,
-                     distance + furthestDescendantDistance);
+  return RangeType<ElemType>(
+      std::max(distance - furthestDescendantDistance, 0.0),
+      distance + furthestDescendantDistance);
 }
 
 //! For a newly initialized node, create children using the near and far set.
@@ -1032,8 +1187,8 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CreateChildren(
   }
 
   const int nextScale = std::min(scale,
-      (int) ceil(log(maxDistance) / log(base))) - 1;
-  const ElemType bound = pow(base, nextScale);
+      (int) std::ceil(std::log(maxDistance) / std::log(base))) - 1;
+  const ElemType bound = std::pow(base, nextScale);
 
   // First, make the self child.  We must split the given near set into the near
   // set and far set for the self child.
@@ -1525,11 +1680,10 @@ inline void CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
     // Now take its child.
     children.push_back(&(old->Child(0)));
 
-    // Set its parent and parameters correctly, and rebuild the statistic.
+    // Set its parent and parameters correctly.
     old->Child(0).Parent() = this;
     old->Child(0).ParentDistance() = old->ParentDistance();
     old->Child(0).DistanceComps() = old->DistanceComps();
-    old->Child(0).Stat() = StatisticType(old->Child(0));
 
     // Remove its child (so it doesn't delete it).
     old->Children().erase(old->Children().begin() + old->Children().size() - 1);
@@ -1540,7 +1694,7 @@ inline void CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::
 }
 
 /**
- * Default constructor, only for use with boost::serialization.
+ * Default constructor, only for use with cereal.
  */
 template<
     typename MetricType,
@@ -1559,13 +1713,14 @@ CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::CoverTree() :
     furthestDescendantDistance(0.0),
     localMetric(false),
     localDataset(false),
-    metric(NULL)
+    metric(NULL),
+    distanceComps(0)
 {
   // Nothing to do.
 }
 
 /**
- * Serialize to/from a boost::serialization archive.
+ * Serialize to/from a cereal archive.
  */
 template<
     typename MetricType,
@@ -1574,15 +1729,13 @@ template<
     typename RootPointPolicy
 >
 template<typename Archive>
-void CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::Serialize(
+void CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::serialize(
     Archive& ar,
-    const unsigned int /* version */)
+    const uint32_t /* version */)
 {
-  using data::CreateNVP;
-
   // If we're loading, and we have children, they need to be deleted.  We may
   // also need to delete the local metric and dataset.
-  if (Archive::is_loading::value)
+  if (cereal::is_loading<Archive>())
   {
     for (size_t i = 0; i < children.size(); ++i)
       delete children[i];
@@ -1591,53 +1744,35 @@ void CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::Serialize(
       delete metric;
     if (localDataset && dataset)
       delete dataset;
+
+    parent = NULL;
   }
 
-  ar & CreateNVP(dataset, "dataset");
-  ar & CreateNVP(point, "point");
-  ar & CreateNVP(scale, "scale");
-  ar & CreateNVP(base, "base");
-  ar & CreateNVP(stat, "stat");
-  ar & CreateNVP(numDescendants, "numDescendants");
+  bool hasParent = (parent != NULL);
+  ar(CEREAL_NVP(hasParent));
+  MatType*& datasetTemp = const_cast<MatType*&>(dataset);
+  if (!hasParent)
+    ar(CEREAL_POINTER(datasetTemp));
 
-  // Due to quirks of boost::serialization, depending on how the user
-  // serializes the tree, it's possible that the root of the tree will
-  // accidentally be serialized twice.  So if we are a first-level child, we
-  // avoid serializing the parent.  The true (non-duplicated) parent will fix
-  // the parent link.
-  if (Archive::is_saving::value && parent != NULL && parent->Parent() == NULL)
-  {
-    CoverTree* fakeParent = NULL;
-    ar & CreateNVP(fakeParent, "parent");
-  }
-  else
-  {
-    ar & CreateNVP(parent, "parent");
-  }
+  ar(CEREAL_NVP(point));
+  ar(CEREAL_NVP(scale));
+  ar(CEREAL_NVP(base));
+  ar(CEREAL_NVP(stat));
+  ar(CEREAL_NVP(numDescendants));
+  ar(CEREAL_NVP(parentDistance));
+  ar(CEREAL_NVP(furthestDescendantDistance));
+  ar(CEREAL_POINTER(metric));
 
-  ar & CreateNVP(parentDistance, "parentDistance");
-  ar & CreateNVP(furthestDescendantDistance, "furthestDescendantDistance");
-  ar & CreateNVP(metric, "metric");
-
-  if (Archive::is_loading::value && parent == NULL)
+  if (cereal::is_loading<Archive>() && !hasParent)
   {
     localMetric = true;
     localDataset = true;
   }
 
   // Lastly, serialize the children.
-  size_t numChildren = children.size();
-  ar & CreateNVP(numChildren, "numChildren");
-  if (Archive::is_loading::value)
-    children.resize(numChildren);
-  for (size_t i = 0; i < numChildren; ++i)
-  {
-    std::ostringstream oss;
-    oss << "child" << i;
-    ar & CreateNVP(children[i], oss.str());
-  }
+  ar(CEREAL_VECTOR_POINTER(children));
 
-  if (Archive::is_loading::value && parent == NULL)
+  if (cereal::is_loading<Archive>())
   {
     // Look through each child individually.
     for (size_t i = 0; i < children.size(); ++i)
@@ -1647,9 +1782,27 @@ void CoverTree<MetricType, StatisticType, MatType, RootPointPolicy>::Serialize(
       children[i]->Parent() = this;
     }
   }
+
+  if (!hasParent)
+  {
+    std::stack<CoverTree*> stack;
+    for (size_t i = 0; i < children.size(); ++i)
+    {
+      stack.push(children[i]);
+    }
+    while (!stack.empty())
+    {
+      CoverTree* node = stack.top();
+      stack.pop();
+      node->dataset = dataset;
+      for (size_t i = 0; i < node->children.size(); ++i)
+      {
+        stack.push(node->children[i]);
+      }
+    }
+  }
 }
 
-} // namespace tree
 } // namespace mlpack
 
 #endif
