@@ -1,5 +1,5 @@
 /**
- * @file fastmks_stat.hpp
+ * @file methods/fastmks/fastmks_stat.hpp
  * @author Ryan Curtin
  *
  * The statistic used in trees with FastMKS.
@@ -16,7 +16,6 @@
 #include <mlpack/core/tree/tree_traits.hpp>
 
 namespace mlpack {
-namespace fastmks {
 
 /**
  * The statistic used in trees with FastMKS.  This stores both the bound and the
@@ -49,12 +48,12 @@ class FastMKSStat
       lastKernelNode(NULL)
   {
     // Do we have to calculate the centroid?
-    if (tree::TreeTraits<TreeType>::FirstPointIsCentroid)
+    if (TreeTraits<TreeType>::FirstPointIsCentroid)
     {
       // If this type of tree has self-children, then maybe the evaluation is
       // already done.  These statistics are built bottom-up, so the child stat
       // should already be done.
-      if ((tree::TreeTraits<TreeType>::HasSelfChildren) &&
+      if ((TreeTraits<TreeType>::HasSelfChildren) &&
           (node.NumChildren() > 0) &&
           (node.Point(0) == node.Child(0).Point(0)))
       {
@@ -62,7 +61,7 @@ class FastMKSStat
       }
       else
       {
-        selfKernel = sqrt(node.Metric().Kernel().Evaluate(
+        selfKernel = std::sqrt(node.Metric().Kernel().Evaluate(
             node.Dataset().col(node.Point(0)),
             node.Dataset().col(node.Point(0))));
       }
@@ -73,7 +72,7 @@ class FastMKSStat
       arma::vec center;
       node.Center(center);
 
-      selfKernel = sqrt(node.Metric().Kernel().Evaluate(center, center));
+      selfKernel = std::sqrt(node.Metric().Kernel().Evaluate(center, center));
     }
   }
 
@@ -100,13 +99,13 @@ class FastMKSStat
 
   //! Serialize the statistic.
   template<typename Archive>
-  void Serialize(Archive& ar, const unsigned int /* version */)
+  void serialize(Archive& ar, const uint32_t /* version */)
   {
-    ar & data::CreateNVP(bound, "bound");
-    ar & data::CreateNVP(selfKernel, "selfKernel");
+    ar(CEREAL_NVP(bound));
+    ar(CEREAL_NVP(selfKernel));
 
     // Void out last kernel information on load.
-    if (Archive::is_loading::value)
+    if (cereal::is_loading<Archive>())
     {
       lastKernel = 0.0;
       lastKernelNode = NULL;
@@ -128,7 +127,6 @@ class FastMKSStat
   void* lastKernelNode;
 };
 
-} // namespace fastmks
 } // namespace mlpack
 
 #endif
