@@ -1,5 +1,5 @@
 /**
- * @file hmm_train_main.cpp
+ * @file methods/hmm/hmm_train_main.cpp
  * @author Ryan Curtin
  *
  * Executable which trains an HMM and saves the trained HMM to file.
@@ -9,38 +9,70 @@
  * 3-clause BSD license along with mlpack.  If not, see
  * http://www.opensource.org/licenses/BSD-3-Clause for more information.
  */
-#include <mlpack/prereqs.hpp>
-#include <mlpack/core/data/load.hpp>
-#include <mlpack/core/data/save.hpp>
+#include <mlpack/core.hpp>
+
+#undef BINDING_NAME
+#define BINDING_NAME hmm_train
+
+#include <mlpack/core/util/mlpack_main.hpp>
 
 #include "hmm.hpp"
-#include "hmm_util.hpp"
+#include "hmm_model.hpp"
 
 #include <mlpack/methods/gmm/gmm.hpp>
 
-PROGRAM_INFO("Hidden Markov Model (HMM) Training", "This program allows a "
-    "Hidden Markov Model to be trained on labeled or unlabeled data.  It "
-    "support three types of HMMs: discrete HMMs, Gaussian HMMs, or GMM HMMs."
+using namespace mlpack;
+using namespace mlpack::util;
+using namespace arma;
+using namespace std;
+
+// Program Name.
+BINDING_USER_NAME("Hidden Markov Model (HMM) Training");
+
+// Short description.
+BINDING_SHORT_DESC(
+    "An implementation of training algorithms for Hidden Markov Models (HMMs). "
+    "Given labeled or unlabeled data, an HMM can be trained for further use "
+    "with other mlpack HMM tools.");
+
+// Long description.
+BINDING_LONG_DESC(
+    "This program allows a Hidden Markov Model to be trained on labeled or "
+    "unlabeled data.  It supports four types of HMMs: Discrete HMMs, "
+    "Gaussian HMMs, GMM HMMs, or Diagonal GMM HMMs"
     "\n\n"
-    "Either one input sequence can be specified (with --input_file), or, a "
-    "file containing files in which input sequences can be found (when "
-    "--input_file and --batch are used together).  In addition, labels can be "
-    "provided in the file specified by --labels_file, and if --batch is used, "
-    "the file given to --labels_file should contain a list of files of labels "
-    "corresponding to the sequences in the file given to --input_file."
+    "Either one input sequence can be specified (with " +
+    PRINT_PARAM_STRING("input_file") + "), or, a file containing files in "
+    "which input sequences can be found (when "+
+    PRINT_PARAM_STRING("input_file") + "and" + PRINT_PARAM_STRING("batch") +
+    " are used together).  In addition, labels can be "
+    "provided in the file specified by " + PRINT_PARAM_STRING("labels_file") +
+    ", and if " + PRINT_PARAM_STRING("batch") + " is used, "
+    "the file given to " + PRINT_PARAM_STRING("labels_file") +
+    " should contain a list of files of labels corresponding to the sequences"
+    " in the file given to " + PRINT_PARAM_STRING("input_file") + "."
     "\n\n"
     "The HMM is trained with the Baum-Welch algorithm if no labels are "
     "provided.  The tolerance of the Baum-Welch algorithm can be set with the "
-    "--tolerance option.  By default, the transition matrix is randomly "
-    "initialized and the emission distributions are initialized to fit the "
-    "extent of the data."
+    + PRINT_PARAM_STRING("tolerance") + "option.  By default, the transition "
+    "matrix is randomly initialized and the emission distributions are "
+    "initialized to fit the extent of the data."
     "\n\n"
     "Optionally, a pre-created HMM model can be used as a guess for the "
-    "transition matrix and emission probabilities; this is specifiable with "
-    "--model_file.");
+    "transition matrix and emission probabilities; this is specifiable with " +
+    PRINT_PARAM_STRING("output_model") + ".");
+
+// See also...
+BINDING_SEE_ALSO("@hmm_generate", "#hmm_generate");
+BINDING_SEE_ALSO("@hmm_loglik", "#hmm_loglik");
+BINDING_SEE_ALSO("@hmm_viterbi", "#hmm_viterbi");
+BINDING_SEE_ALSO("Hidden Mixture Models on Wikipedia",
+    "https://en.wikipedia.org/wiki/Hidden_Markov_model");
+BINDING_SEE_ALSO("HMM class documentation", "@src/mlpack/methods/hmm/hmm.hpp");
 
 PARAM_STRING_IN_REQ("input_file", "File containing input observations.", "i");
-PARAM_STRING_IN_REQ("type", "Type of HMM: discrete | gaussian | gmm.", "t");
+PARAM_STRING_IN("type", "Type of HMM: discrete | gaussian | diag_gmm | gmm.",
+    "t", "gaussian");
 
 PARAM_FLAG("batch", "If true, input_file (and if passed, labels_file) are "
     "expected to contain a list of files to use as input observation sequences "
@@ -49,43 +81,36 @@ PARAM_INT_IN("states", "Number of hidden states in HMM (necessary, unless "
     "model_file is specified).", "n", 0);
 PARAM_INT_IN("gaussians", "Number of gaussians in each GMM (necessary when type"
     " is 'gmm').", "g", 0);
-PARAM_STRING_IN("model_file", "Pre-existing HMM model file.", "m", "");
+PARAM_MODEL_IN(HMMModel, "input_model", "Pre-existing HMM model to initialize "
+    "training with.", "m");
 PARAM_STRING_IN("labels_file", "Optional file of hidden states, used for "
     "labeled training.", "l", "");
-PARAM_STRING_OUT("output_model_file", "File to save trained HMM to.", "o");
+PARAM_MODEL_OUT(HMMModel, "output_model", "Output for trained HMM.", "M");
 PARAM_INT_IN("seed", "Random seed.  If 0, 'std::time(NULL)' is used.", "s", 0);
 PARAM_DOUBLE_IN("tolerance", "Tolerance of the Baum-Welch algorithm.", "T",
     1e-5);
 
-using namespace mlpack;
-using namespace mlpack::hmm;
-using namespace mlpack::distribution;
-using namespace mlpack::util;
-using namespace mlpack::gmm;
-using namespace mlpack::math;
-using namespace arma;
-using namespace std;
-
 // Because we don't know what the type of our HMM is, we need to write a
 // function that can take arbitrary HMM types.
 struct Init
 {
   template<typename HMMType>
-  static void Apply(HMMType& hmm, vector<mat>* trainSeq)
+  static void Apply(util::Params& params, HMMType& hmm, vector<mat>* trainSeq)
   {
-    const size_t states = CLI::GetParam<int>("states");
-    const double tolerance = CLI::GetParam<double>("tolerance");
+    const size_t states = params.Get<int>("states");
+    const double tolerance = params.Get<double>("tolerance");
 
     // Create the initialized-to-zero model.
-    Create(hmm, *trainSeq, states, tolerance);
+    Create(params, hmm, *trainSeq, states, tolerance);
 
     // Initializing the emission distribution depends on the distribution.
     // Therefore we have to use the helper functions.
-    RandomInitialize(hmm.Emission());
+    RandomInitialize(params, hmm.Emission());
   }
 
   //! Helper function to create discrete HMM.
-  static void Create(HMM<DiscreteDistribution>& hmm,
+  static void Create(util::Params& /* params */,
+                     HMM<DiscreteDistribution>& hmm,
                      vector<mat>& trainSeq,
                      size_t states,
                      double tolerance)
@@ -98,7 +123,7 @@ struct Init
          ++it)
     {
       arma::Col<size_t> maxSeqs =
-          arma::conv_to<arma::Col<size_t>>::from(arma::max(*it, 1)) + 1;
+          ConvTo<arma::Col<size_t>>::From(arma::max(*it, 1)) + 1;
       maxEmissions = arma::max(maxEmissions, maxSeqs);
     }
 
@@ -107,7 +132,8 @@ struct Init
   }
 
   //! Helper function to create Gaussian HMM.
-  static void Create(HMM<GaussianDistribution>& hmm,
+  static void Create(util::Params& /* params */,
+                     HMM<GaussianDistribution>& hmm,
                      vector<mat>& trainSeq,
                      size_t states,
                      double tolerance)
@@ -117,10 +143,14 @@ struct Init
 
     // Verify dimensionality of data.
     for (size_t i = 0; i < trainSeq.size(); ++i)
+    {
       if (trainSeq[i].n_rows != dimensionality)
+      {
         Log::Fatal << "Observation sequence " << i << " dimensionality ("
             << trainSeq[i].n_rows << " is incorrect (should be "
             << dimensionality << ")!" << endl;
+      }
+    }
 
     // Get the model and initialize it.
     hmm = HMM<GaussianDistribution>(size_t(states),
@@ -128,78 +158,148 @@ struct Init
   }
 
   //! Helper function to create GMM HMM.
-  static void Create(HMM<GMM>& hmm,
+  static void Create(util::Params& params,
+                     HMM<GMM>& hmm,
                      vector<mat>& trainSeq,
                      size_t states,
                      double tolerance)
   {
     // Find dimension of the data.
     const size_t dimensionality = trainSeq[0].n_rows;
-    const int gaussians = CLI::GetParam<int>("gaussians");
+    const int gaussians = params.Get<int>("gaussians");
 
     if (gaussians == 0)
-      Log::Fatal << "Number of gaussians for each GMM must be specified (-g) "
+    {
+      Log::Fatal << "Number of gaussians for each GMM must be specified "
           << "when type = 'gmm'!" << endl;
+    }
 
     if (gaussians < 0)
+    {
       Log::Fatal << "Invalid number of gaussians (" << gaussians << "); must "
           << "be greater than or equal to 1." << endl;
+    }
 
     // Create HMM object.
     hmm = HMM<GMM>(size_t(states), GMM(size_t(gaussians), dimensionality),
         tolerance);
 
     // Issue a warning if the user didn't give labels.
-    if (!CLI::HasParam("labels_file"))
+    if (!params.Has("labels_file"))
+    {
       Log::Warn << "Unlabeled training of GMM HMMs is almost certainly not "
           << "going to produce good results!" << endl;
+    }
+  }
+
+  //! Helper function to create Diagonal GMM HMM.
+  static void Create(util::Params& params,
+                     HMM<DiagonalGMM>& hmm,
+                     vector<mat>& trainSeq,
+                     size_t states,
+                     double tolerance)
+  {
+    // Find dimension of the data.
+    const size_t dimensionality = trainSeq[0].n_rows;
+    const int gaussians = params.Get<int>("gaussians");
+
+    if (gaussians == 0)
+    {
+      Log::Fatal << "Number of gaussians for each GMM must be specified "
+          << "when type = 'diag_gmm'!" << endl;
+    }
+
+    if (gaussians < 0)
+    {
+      Log::Fatal << "Invalid number of gaussians (" << gaussians << "); must "
+          << "be greater than or equal to 1." << endl;
+    }
+
+    // Create HMM object.
+    hmm = HMM<DiagonalGMM>(size_t(states), DiagonalGMM(size_t(gaussians),
+        dimensionality), tolerance);
+
+    // Issue a warning if the user didn't give labels.
+    if (!params.Has("labels_file"))
+    {
+      Log::Warn << "Unlabeled training of Diagonal GMM HMMs is almost "
+          << "certainly not going to produce good results!" << endl;
+    }
   }
 
   //! Helper function for discrete emission distributions.
-  static void RandomInitialize(vector<DiscreteDistribution>& e)
+  static void RandomInitialize(util::Params& /* params */,
+                               vector<DiscreteDistribution>& e)
   {
     for (size_t i = 0; i < e.size(); ++i)
     {
       e[i].Probabilities().randu();
-      e[i].Probabilities() /= arma::accu(e[i].Probabilities());
+      e[i].Probabilities() /= accu(e[i].Probabilities());
     }
   }
 
   //! Helper function for Gaussian emission distributions.
-  static void RandomInitialize(vector<GaussianDistribution>& e)
+  static void RandomInitialize(util::Params& /* params */,
+                               vector<GaussianDistribution>& e)
   {
     for (size_t i = 0; i < e.size(); ++i)
     {
       const size_t dimensionality = e[i].Mean().n_rows;
       e[i].Mean().randu();
       // Generate random covariance.
-      arma::mat r = arma::randu<arma::mat>(dimensionality, dimensionality);
+      arma::mat r;
+      r.randu(dimensionality, dimensionality);
       e[i].Covariance(r * r.t());
     }
   }
 
   //! Helper function for GMM emission distributions.
-  static void RandomInitialize(vector<GMM>& e)
+  static void RandomInitialize(util::Params& params,
+                               vector<GMM>& e)
   {
     for (size_t i = 0; i < e.size(); ++i)
     {
       // Random weights.
       e[i].Weights().randu();
-      e[i].Weights() /= arma::accu(e[i].Weights());
+      e[i].Weights() /= accu(e[i].Weights());
 
       // Random means and covariances.
-      for (int g = 0; g < CLI::GetParam<int>("gaussians"); ++g)
+      for (int g = 0; g < params.Get<int>("gaussians"); ++g)
       {
         const size_t dimensionality = e[i].Component(g).Mean().n_rows;
         e[i].Component(g).Mean().randu();
 
         // Generate random covariance.
-        arma::mat r = arma::randu<arma::mat>(dimensionality,
-            dimensionality);
+        arma::mat r;
+        r.randu(dimensionality, dimensionality);
         e[i].Component(g).Covariance(r * r.t());
       }
     }
   }
+
+  //! Helper function for Diagonal GMM emission distributions.
+  static void RandomInitialize(util::Params& params,
+                               vector<DiagonalGMM>& e)
+  {
+    for (size_t i = 0; i < e.size(); ++i)
+    {
+      // Random weights.
+      e[i].Weights().randu();
+      e[i].Weights() /= accu(e[i].Weights());
+
+      // Random means and covariances.
+      for (int g = 0; g < params.Get<int>("gaussians"); ++g)
+      {
+        const size_t dimensionality = e[i].Component(g).Mean().n_rows;
+        e[i].Component(g).Mean().randu();
+
+        // Generate random diagonal covariance.
+        arma::vec r;
+        r.randu(dimensionality);
+        e[i].Component(g).Covariance(r);
+      }
+    }
+  }
 };
 
 // Because we don't know what the type of our HMM is, we need to write a
@@ -207,29 +307,35 @@ struct Init
 struct Train
 {
   template<typename HMMType>
-  static void Apply(HMMType& hmm, vector<mat>* trainSeqPtr)
+  static void Apply(util::Params& params,
+                    HMMType& hmm,
+                    vector<mat>* trainSeqPtr)
   {
-    const bool batch = CLI::HasParam("batch");
-    const double tolerance = CLI::GetParam<double>("tolerance");
+    const bool batch = params.Has("batch");
+    const double tolerance = params.Get<double>("tolerance");
 
     // Do we need to replace the tolerance?
-    if (CLI::HasParam("tolerance"))
+    if (params.Has("tolerance"))
       hmm.Tolerance() = tolerance;
 
-    const string labelsFile = CLI::GetParam<string>("labels_file");
+    const string labelsFile = params.Get<string>("labels_file");
 
     // Verify that the dimensionality of our observations is the same as the
     // dimensionality of our HMM's emissions.
     vector<mat>& trainSeq = *trainSeqPtr;
     for (size_t i = 0; i < trainSeq.size(); ++i)
+    {
       if (trainSeq[i].n_rows != hmm.Emission()[0].Dimensionality())
+      {
         Log::Fatal << "Dimensionality of training sequence " << i << " ("
             << trainSeq[i].n_rows << ") is not equal to the dimensionality of "
             << "the HMM (" << hmm.Emission()[0].Dimensionality() << ")!"
             << endl;
+      }
+    }
 
     vector<arma::Row<size_t>> labelSeq; // May be empty.
-    if (CLI::HasParam("labels_file"))
+    if (params.Has("labels_file"))
     {
       // Do we have multiple label files to load?
       char lineBuf[1024];
@@ -292,9 +398,11 @@ struct Train
 
         // Verify the same number of observations as the data.
         if (label.n_elem != trainSeq[labelSeq.size()].n_cols)
+        {
           Log::Fatal << "Label sequence " << labelSeq.size() << " does not have"
               << " the same number of points as observation sequence "
               << labelSeq.size() << "!" << endl;
+        }
 
         // Check all of the labels.
         for (size_t i = 0; i < label.n_cols; ++i)
@@ -319,52 +427,50 @@ struct Train
       // Perform unsupervised training.
       hmm.Train(trainSeq);
     }
-
-    // Save the model.
-    if (CLI::HasParam("output_model_file"))
-    {
-      const string modelFile = CLI::GetParam<string>("output_model_file");
-      SaveHMM(hmm, modelFile);
-    }
   }
 };
 
-int main(int argc, char** argv)
+void BINDING_FUNCTION(util::Params& params, util::Timers& /* timers */)
 {
-  // Parse command line options.
-  CLI::ParseCommandLine(argc, argv);
-
   // Set random seed.
-  if (CLI::GetParam<int>("seed") != 0)
-    RandomSeed((size_t) CLI::GetParam<int>("seed"));
+  if (params.Get<int>("seed") != 0)
+    RandomSeed((size_t) params.Get<int>("seed"));
   else
     RandomSeed((size_t) time(NULL));
 
   // Validate parameters.
-  const string modelFile = CLI::GetParam<string>("model_file");
-  const string inputFile = CLI::GetParam<string>("input_file");
-  const string type = CLI::GetParam<string>("type");
-  const size_t states = CLI::GetParam<int>("states");
-  const double tolerance = CLI::GetParam<double>("tolerance");
-  const bool batch = CLI::HasParam("batch");
-
-  // Verify that either a model or a type was given.
-  if (modelFile == "" && type == "")
-    Log::Fatal << "No model file specified and no HMM type given!  At least "
-        << "one is required." << endl;
+  const string inputFile = params.Get<string>("input_file");
+  const string type = params.Get<string>("type");
+  const bool batch = params.Has("batch");
+  const double tolerance = params.Get<double>("tolerance");
 
   // If no model is specified, make sure we are training with valid parameters.
-  if (modelFile == "")
+  if (!params.Has("input_model"))
   {
     // Validate number of states.
-    if (states == 0)
-      Log::Fatal << "Must specify number of states if model file is not "
-          << "specified!" << endl;
+    RequireAtLeastOnePassed(params, { "states" }, true);
+    RequireAtLeastOnePassed(params, { "type" }, true);
+    RequireParamValue<int>(params, "states", [](int x) { return x > 0; }, true,
+        "number of states must be positive");
   }
 
-  if (modelFile != "" && CLI::HasParam("tolerance"))
-    Log::Info << "Tolerance of existing model in '" << modelFile << "' will be "
+  if (params.Has("input_model") && params.Has("tolerance"))
+  {
+    Log::Info << "Tolerance of existing model in '"
+        << params.GetPrintable<HMMModel*>("input_model") << "' will be "
         << "replaced with specified tolerance of " << tolerance << "." << endl;
+  }
+
+  ReportIgnoredParam(params, {{ "input_model", true }}, "type");
+
+  if (!params.Has("input_model"))
+  {
+    RequireParamInSet<string>(params, "type", { "discrete", "gaussian", "gmm",
+        "diag_gmm" }, true, "unknown HMM type");
+  }
+
+  RequireParamValue<double>(params, "tolerance",
+      [](double x) { return x >= 0; }, true, "tolerance must be non-negative");
 
   // Load the input data.
   vector<mat> trainSeq;
@@ -377,8 +483,10 @@ int main(int argc, char** argv)
     fstream f(inputFile.c_str(), ios_base::in);
 
     if (!f.is_open())
+    {
       Log::Fatal << "Could not open '" << inputFile << "' for reading."
           << endl;
+    }
 
     // Now read each line in.
     char lineBuf[1024]; // Max 1024 characters... hopefully long enough.
@@ -411,182 +519,43 @@ int main(int argc, char** argv)
     data::Load(inputFile, trainSeq[0], true);
   }
 
+  // Get the type.
+  HMMType typeId;
+  if (type == "discrete")
+    typeId = HMMType::DiscreteHMM;
+  else if (type == "gaussian")
+    typeId = HMMType::GaussianHMM;
+  else if (type == "gmm")
+    typeId = HMMType::GaussianMixtureModelHMM;
+  else
+    typeId = HMMType::DiagonalGaussianMixtureModelHMM;
+
   // If we have a model file, we can autodetect the type.
-  if (CLI::HasParam("model_file"))
+  HMMModel* hmm;
+  if (params.Has("input_model"))
   {
-    LoadHMMAndPerformAction<Train>(modelFile, &trainSeq);
+    hmm = params.Get<HMMModel*>("input_model");
+
+    hmm->PerformAction<Train, vector<mat>>(params, &trainSeq);
   }
   else
   {
-    // We need to read in the type and build the HMM by hand.
-    const string type = CLI::GetParam<string>("type");
+    // We need to initialize the model.
+    hmm = new HMMModel(typeId);
 
-    if (type == "discrete")
+    // Catch any exceptions so that we can clean the model if needed.
+    try
     {
-      // Maximum observation is necessary so we know how to train the discrete
-      // distribution.
-      size_t maxEmission = 0;
-      for (vector<mat>::iterator it = trainSeq.begin(); it != trainSeq.end();
-           ++it)
-      {
-        size_t maxSeq = size_t(as_scalar(max(trainSeq[0], 1))) + 1;
-        if (maxSeq > maxEmission)
-          maxEmission = maxSeq;
-      }
-
-      Log::Info << maxEmission << " discrete observations in the input data."
-          << endl;
-
-      // Create HMM object.
-      HMM<DiscreteDistribution> hmm(size_t(states),
-          DiscreteDistribution(maxEmission), tolerance);
-
-      // Initialize emissions using the distribution of the full data.
-      DiscreteDistribution sampleEmission;
-      if (trainSeq.size() > 1)
-      {
-        // Flatten data matrix for training of an emission distribution.  This
-        // is not efficient!
-        size_t totalCols = 0;
-        for (size_t i = 0; i < trainSeq.size(); ++i)
-          totalCols += trainSeq[i].n_cols;
-
-        arma::mat flatData(trainSeq[0].n_rows, totalCols);
-        size_t currentCol = 0;
-        for (size_t i = 0; i < trainSeq.size(); ++i)
-        {
-          flatData.cols(currentCol, currentCol + trainSeq[i].n_cols - 1) =
-              trainSeq[i];
-          currentCol += trainSeq[i].n_cols;
-        }
-
-        sampleEmission.Train(flatData);
-      }
-      else
-      {
-        sampleEmission.Train(trainSeq[0]);
-      }
-
-      // Apply initialized emissions.
-      for (size_t e = 0; e < hmm.Transition().n_cols; ++e)
-        hmm.Emission()[e] = sampleEmission;
-
-      // Now train it.  Pass the already-loaded training data.
-      Train::Apply(hmm, &trainSeq);
+      hmm->PerformAction<Init, vector<mat>>(params, &trainSeq);
+      hmm->PerformAction<Train, vector<mat>>(params, &trainSeq);
     }
-    else if (type == "gaussian")
+    catch (std::exception& e)
     {
-      // Find dimension of the data.
-      const size_t dimensionality = trainSeq[0].n_rows;
-
-      // Verify dimensionality of data.
-      for (size_t i = 0; i < trainSeq.size(); ++i)
-        if (trainSeq[i].n_rows != dimensionality)
-          Log::Fatal << "Observation sequence " << i << " dimensionality ("
-              << trainSeq[i].n_rows << " is incorrect (should be "
-              << dimensionality << ")!" << endl;
-
-      HMM<GaussianDistribution> hmm(size_t(states),
-          GaussianDistribution(dimensionality), tolerance);
-
-      // Initialize emissions using the distribution of the full data.
-      GaussianDistribution sampleEmission;
-      if (trainSeq.size() > 1)
-      {
-        // Flatten data matrix for training of an emission distribution.  This
-        // is not efficient!
-        size_t totalCols = 0;
-        for (size_t i = 0; i < trainSeq.size(); ++i)
-          totalCols += trainSeq[i].n_cols;
-
-        arma::mat flatData(trainSeq[0].n_rows, totalCols);
-        size_t currentCol = 0;
-        for (size_t i = 0; i < trainSeq.size(); ++i)
-        {
-          flatData.cols(currentCol, currentCol + trainSeq[i].n_cols - 1) =
-              trainSeq[i];
-          currentCol += trainSeq[i].n_cols;
-        }
-
-        sampleEmission.Train(flatData);
-      }
-      else
-      {
-        sampleEmission.Train(trainSeq[0]);
-      }
-
-      // Set all emissions to the initialized emission.
-      for (size_t e = 0; e < hmm.Transition().n_cols; ++e)
-        hmm.Emission()[e] = sampleEmission;
-
-      // Now train it.
-      Train::Apply(hmm, &trainSeq);
-    }
-    else if (type == "gmm")
-    {
-      // Find dimension of the data.
-      const size_t dimensionality = trainSeq[0].n_rows;
-
-      const int gaussians = CLI::GetParam<int>("gaussians");
-
-      if (gaussians == 0)
-        Log::Fatal << "Number of gaussians for each GMM must be specified (-g) "
-            << "when type = 'gmm'!" << endl;
-
-      if (gaussians < 0)
-        Log::Fatal << "Invalid number of gaussians (" << gaussians << "); must "
-            << "be greater than or equal to 1." << endl;
-
-      // Create HMM object.
-      HMM<GMM> hmm(size_t(states), GMM(size_t(gaussians), dimensionality),
-          tolerance);
-
-      // Initialize emissions using the distribution of the full data.
-      // Super-simple emission training: we don't want it to take long at all.
-      GMM sampleEmission;
-      EMFit<> fitter(1, 0.01); // Only one iteration of EM GMM training.
-      if (trainSeq.size() > 1)
-      {
-        // Flatten data matrix for training of an emission distribution.  This
-        // is not efficient!
-        size_t totalCols = 0;
-        for (size_t i = 0; i < trainSeq.size(); ++i)
-          totalCols += trainSeq[i].n_cols;
-
-        arma::mat flatData(trainSeq[0].n_rows, totalCols);
-        size_t currentCol = 0;
-        for (size_t i = 0; i < trainSeq.size(); ++i)
-        {
-          flatData.cols(currentCol, currentCol + trainSeq[i].n_cols - 1) =
-              trainSeq[i];
-          currentCol += trainSeq[i].n_cols;
-        }
-
-        sampleEmission.Train(flatData, 1, false, fitter);
-      }
-      else
-      {
-        sampleEmission.Train(trainSeq[0], 1, false, fitter);
-      }
-
-      // Set all emissions to the initialized emission.
-      for (size_t e = 0; e < hmm.Transition().n_cols; ++e)
-        hmm.Emission()[e] = sampleEmission;
-
-      // Now train it.
-      Train::Apply(hmm, &trainSeq);
-
-      // Issue a warning if the user didn't give labels.
-      if (!CLI::HasParam("labels_file"))
-        Log::Warn << "Unlabeled training of GMM HMMs is almost certainly not "
-            << "going to produce good results!" << endl;
-
-      Train::Apply(hmm, &trainSeq);
-    }
-    else
-    {
-      Log::Fatal << "Unknown HMM type: " << type << "; must be 'discrete', "
-          << "'gaussian', or 'gmm'." << endl;
+      delete hmm;
+      throw;
     }
   }
+
+  // If necessary, save the output.
+  params.Get<HMMModel*>("output_model") = hmm;
 }
