@@ -1,11 +1,9 @@
 /**
- * @file elkan_kmeans.hpp
+ * @file methods/kmeans/elkan_kmeans.hpp
  * @author Ryan Curtin
  *
  * An implementation of Elkan's algorithm for exact Lloyd iterations.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -15,7 +13,6 @@
 #define MLPACK_METHODS_KMEANS_ELKAN_KMEANS_HPP
 
 namespace mlpack {
-namespace kmeans {
 
 template<typename MetricType, typename MatType>
 class ElkanKMeans
@@ -63,7 +60,6 @@ class ElkanKMeans
   size_t distanceCalculations;
 };
 
-} // namespace kmeans
 } // namespace mlpack
 
 // Include implementation.
