@@ -1,38 +1,64 @@
 /**
- * @file radical_main.cpp
+ * @file methods/radical/radical_main.cpp
  * @author Nishant Mehta
  *
- * Executable for RADICAL.
+ * Executable for RADICAL. RADICAL is Robust, Accurate, Direct ICA
+ * aLgorithm.
  *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
  * http://www.opensource.org/licenses/BSD-3-Clause for more information.
  */
-#include <mlpack/prereqs.hpp>
-#include <mlpack/core/util/cli.hpp>
+#include <mlpack/core.hpp>
+
+#undef BINDING_NAME
+#define BINDING_NAME radical
+
 #include <mlpack/core/util/mlpack_main.hpp>
-#include <mlpack/core/math/random.hpp>
 #include "radical.hpp"
 
-PROGRAM_INFO("RADICAL", "An implementation of RADICAL, a method for independent"
-    "component analysis (ICA).  Assuming that we have an input matrix X, the"
-    "goal is to find a square unmixing matrix W such that Y = W * X and the "
-    "dimensions of Y are independent components.  If the algorithm is running"
-    "particularly slowly, try reducing the number of replicates."
+// Program Name.
+BINDING_USER_NAME("RADICAL");
+
+// Short description.
+BINDING_SHORT_DESC(
+    "An implementation of RADICAL, a method for independent component analysis "
+    "(ICA).  Given a dataset, this can decompose the dataset into an unmixing "
+    "matrix and an independent component matrix; this can be useful for "
+    "preprocessing.");
+
+// Long description.
+BINDING_LONG_DESC(
+    "An implementation of RADICAL, a method for independent component analysis "
+    "(ICA).  Assuming that we have an input matrix X, the goal is to find a "
+    "square unmixing matrix W such that Y = W * X and the dimensions of Y are "
+    "independent components.  If the algorithm is running particularly slowly, "
+    "try reducing the number of replicates."
     "\n\n"
     "The input matrix to perform ICA on should be specified with the " +
     PRINT_PARAM_STRING("input") + " parameter.  The output matrix Y may be "
     "saved with the " + PRINT_PARAM_STRING("output_ic") + " output parameter, "
     "and the output unmixing matrix W may be saved with the " +
-    PRINT_PARAM_STRING("output_unmixing") + " output parameter."
-    "\n\n"
+    PRINT_PARAM_STRING("output_unmixing") + " output parameter.");
+
+// Example.
+BINDING_EXAMPLE(
     "For example, to perform ICA on the matrix " + PRINT_DATASET("X") + " with "
     "40 replicates, saving the independent components to " +
     PRINT_DATASET("ic") + ", the following command may be used: "
     "\n\n" +
     PRINT_CALL("radical", "input", "X", "replicates", 40, "output_ic", "ic"));
 
+// See also...
+BINDING_SEE_ALSO("Independent component analysis on Wikipedia",
+    "https://en.wikipedia.org/wiki/Independent_component_analysis");
+BINDING_SEE_ALSO("ICA using spacings estimates of entropy (pdf)",
+    "https://www.jmlr.org/papers/volume4/learned-miller03a/"
+    "learned-miller03a.pdf");
+BINDING_SEE_ALSO("Radical C++ class documentation",
+    "@src/mlpack/methods/radical/radical.hpp");
+
 PARAM_MATRIX_IN_REQ("input", "Input dataset for ICA.", "i");
 
 PARAM_MATRIX_OUT("output_ic", "Matrix to save independent components to.", "o");
@@ -51,42 +77,40 @@ PARAM_FLAG("objective", "If set, an estimate of the final objective function "
     "is printed.", "O");
 
 using namespace mlpack;
-using namespace mlpack::radical;
-using namespace mlpack::math;
 using namespace mlpack::util;
 using namespace std;
 using namespace arma;
 
-static void mlpackMain()
+void BINDING_FUNCTION(util::Params& params, util::Timers& timers)
 {
   // Set random seed.
-  if (CLI::GetParam<int>("seed") != 0)
-    RandomSeed((size_t) CLI::GetParam<int>("seed"));
+  if (params.Get<int>("seed") != 0)
+    RandomSeed((size_t) params.Get<int>("seed"));
   else
     RandomSeed((size_t) std::time(NULL));
 
-  RequireAtLeastOnePassed({ "output_ic", "output_unmixing" }, false, "no output"
-      " will be saved");
+  RequireAtLeastOnePassed(params, { "output_ic", "output_unmixing" }, false,
+      "no output will be saved");
 
   // Check validity of parameters.
-  RequireParamValue<int>("replicates", [](int x) { return x > 0; }, true,
-      "number of replicates must be positive");
-  RequireParamValue<double>("noise_std_dev", [](double x) { return x >= 0.0; },
-      true, "standard deviation of Gaussian noise must be greater than or equal"
-      " to 0");
-  RequireParamValue<int>("angles", [](int x) { return x > 0; }, true,
+  RequireParamValue<int>(params, "replicates", [](int x) { return x > 0; },
+      true, "number of replicates must be positive");
+  RequireParamValue<double>(params, "noise_std_dev",
+      [](double x) { return x >= 0.0; }, true, "standard deviation of Gaussian "
+      "noise must be greater than or equal to 0");
+  RequireParamValue<int>(params, "angles", [](int x) { return x > 0; }, true,
       "number of angles must be positive");
-  RequireParamValue<int>("sweeps", [](int x) { return x >= 0; }, true,
+  RequireParamValue<int>(params, "sweeps", [](int x) { return x >= 0; }, true,
       "number of sweeps must be 0 or greater");
 
   // Load the data.
-  mat matX = std::move(CLI::GetParam<mat>("input"));
+  mat matX = std::move(params.Get<mat>("input"));
 
   // Load parameters.
-  double noiseStdDev = CLI::GetParam<double>("noise_std_dev");
-  size_t nReplicates = CLI::GetParam<int>("replicates");
-  size_t nAngles = CLI::GetParam<int>("angles");
-  size_t nSweeps = CLI::GetParam<int>("sweeps");
+  double noiseStdDev = params.Get<double>("noise_std_dev");
+  size_t nReplicates = params.Get<int>("replicates");
+  size_t nAngles = params.Get<int>("angles");
+  size_t nSweeps = params.Get<int>("sweeps");
 
   if (nSweeps == 0)
   {
@@ -97,21 +121,21 @@ static void mlpackMain()
   Radical rad(noiseStdDev, nReplicates, nAngles, nSweeps);
   mat matY;
   mat matW;
-  rad.DoRadical(matX, matY, matW);
+  rad.DoRadical(matX, matY, matW, timers);
 
   // Save results.
-  if (CLI::HasParam("output_ic"))
-    CLI::GetParam<mat>("output_ic") = std::move(matY);
+  if (params.Has("output_ic"))
+    params.Get<mat>("output_ic") = std::move(matY);
 
-  if (CLI::HasParam("output_unmixing"))
-    CLI::GetParam<mat>("output_unmixing") = std::move(matW);
+  if (params.Has("output_unmixing"))
+    params.Get<mat>("output_unmixing") = std::move(matW);
 
-  if (CLI::HasParam("objective"))
+  if (params.Has("objective"))
   {
     // Compute and print objective.
     mat matYT = trans(matY);
     double valEst = 0;
-    for (size_t i = 0; i < matYT.n_cols; i++)
+    for (size_t i = 0; i < matYT.n_cols; ++i)
     {
       vec y = vec(matYT.col(i));
       valEst += rad.Vasicek(y);
