@@ -1,5 +1,5 @@
 /**
- * @file cv_test.cpp
+ * @file tests/cv_test.cpp
  *
  * Unit tests for the cross-validation module.
  *
@@ -8,48 +8,27 @@
  * 3-clause BSD license along with mlpack.  If not, see
  * http://www.opensource.org/licenses/BSD-3-Clause for more information.
  */
+#include <mlpack/core.hpp>
+#include <mlpack/methods/ann.hpp>
+#include <mlpack/methods/decision_tree.hpp>
+#include <mlpack/methods/hoeffding_trees.hpp>
+#include <mlpack/methods/lars.hpp>
+#include <mlpack/methods/linear_regression.hpp>
+#include <mlpack/methods/logistic_regression.hpp>
+#include <mlpack/methods/naive_bayes.hpp>
+#include <mlpack/methods/perceptron.hpp>
+#include <mlpack/methods/softmax_regression.hpp>
 
-#include <type_traits>
-
-#include <mlpack/core/cv/meta_info_extractor.hpp>
-#include <mlpack/core/cv/metrics/accuracy.hpp>
-#include <mlpack/core/cv/metrics/f1.hpp>
-#include <mlpack/core/cv/metrics/mse.hpp>
-#include <mlpack/core/cv/metrics/precision.hpp>
-#include <mlpack/core/cv/metrics/recall.hpp>
-#include <mlpack/core/cv/simple_cv.hpp>
-#include <mlpack/core/cv/k_fold_cv.hpp>
-#include <mlpack/core/optimizers/rmsprop/rmsprop.hpp>
-#include <mlpack/methods/ann/ffn.hpp>
-#include <mlpack/methods/ann/init_rules/const_init.hpp>
-#include <mlpack/methods/ann/layer/layer.hpp>
-#include <mlpack/methods/ann/loss_functions/mean_squared_error.hpp>
-#include <mlpack/methods/decision_tree/decision_tree.hpp>
-#include <mlpack/methods/decision_tree/information_gain.hpp>
-#include <mlpack/methods/hoeffding_trees/hoeffding_tree.hpp>
-#include <mlpack/methods/lars/lars.hpp>
-#include <mlpack/methods/linear_regression/linear_regression.hpp>
-#include <mlpack/methods/logistic_regression/logistic_regression.hpp>
-#include <mlpack/methods/naive_bayes/naive_bayes_classifier.hpp>
-#include <mlpack/methods/softmax_regression/softmax_regression.hpp>
-
-#include <boost/test/unit_test.hpp>
+#include "catch.hpp"
 #include "mock_categorical_data.hpp"
 
 using namespace mlpack;
-using namespace mlpack::ann;
-using namespace mlpack::cv;
-using namespace mlpack::naive_bayes;
-using namespace mlpack::optimization;
-using namespace mlpack::regression;
-using namespace mlpack::tree;
-
-BOOST_AUTO_TEST_SUITE(CVTest);
+using namespace mlpack::data;
 
 /**
  * Test metrics for binary classification.
  */
-BOOST_AUTO_TEST_CASE(BinaryClassificationMetricsTest)
+TEST_CASE("BinaryClassificationMetricsTest", "[CVTest]")
 {
   // Using the same data for training and testing.
   arma::mat data = arma::linspace<arma::rowvec>(1.0, 10.0, 10);
@@ -63,20 +42,115 @@ BOOST_AUTO_TEST_CASE(BinaryClassificationMetricsTest)
 
   LogisticRegression<> lr(data, predictedLabels);
 
-  BOOST_REQUIRE_CLOSE(Accuracy::Evaluate(lr, data, labels), 0.7, 1e-5);
+  REQUIRE(Accuracy::Evaluate(lr, data, labels) == Approx(0.7).epsilon(1e-7));
 
-  BOOST_REQUIRE_CLOSE(Precision<Binary>::Evaluate(lr, data, labels), 0.6, 1e-5);
+  REQUIRE(Precision<Binary>::Evaluate(lr, data, labels)
+          == Approx(0.6).epsilon(1e-7));
 
-  BOOST_REQUIRE_CLOSE(Recall<Binary>::Evaluate(lr, data, labels), 0.75, 1e-5);
+  REQUIRE(Recall<Binary>::Evaluate(lr, data, labels)
+          == Approx(0.75).epsilon(1e-7));
 
   double f1 = 2 * 0.6 * 0.75 / (0.6 + 0.75);
-  BOOST_REQUIRE_CLOSE(F1<Binary>::Evaluate(lr, data, labels), f1, 1e-5);
+  REQUIRE(F1<Binary>::Evaluate(lr, data, labels) == Approx(f1).epsilon(1e-7));
+
+  // Testing binary ROC-AUC Score.
+  //
+  // NOTE:
+  // For comparing these ROCAUCScore testcases with "scikit-learn"
+  // library's "roc_auc_score", refer the pull request thread comment
+  // https://github.com/mlpack/mlpack/pull/3086#issuecomment-1046003548
+  //
+  arma::Row<size_t> rocTrueLabels;
+  arma::Row<double> rocScoresOfPC;
+  double rocAucScore;
+
+  // Test - 1
+  rocTrueLabels = arma::Row<size_t>("0 0 0 0 0  1 1 1 1 1");
+  rocScoresOfPC = arma::Row<double>("0.1 0.2 0.3 0.4 0.5  0.6 0.7 0.8 0.9 1");
+
+  rocAucScore = 0;
+  REQUIRE(ROCAUCScore<0>::Evaluate(rocTrueLabels, rocScoresOfPC)
+          == Approx(rocAucScore).epsilon(1e-7));
+
+  rocAucScore = 1;
+  REQUIRE(ROCAUCScore<1>::Evaluate(rocTrueLabels, rocScoresOfPC)
+          == Approx(rocAucScore).epsilon(1e-7));
+
+  // Test - 2
+  rocTrueLabels = arma::Row<size_t>("1 0 1 0 1  0 1 0 1 0");
+  rocScoresOfPC = arma::Row<double>("0.1 0.2 0.3 0.4 0.5  0.6 0.7 0.8 0.9 1");
+
+  rocAucScore = 0.6;
+  REQUIRE(ROCAUCScore<0>::Evaluate(rocTrueLabels, rocScoresOfPC)
+          == Approx(rocAucScore).epsilon(1e-7));
+
+  rocAucScore = 0.4;
+  REQUIRE(ROCAUCScore<1>::Evaluate(rocTrueLabels, rocScoresOfPC)
+          == Approx(rocAucScore).epsilon(1e-7));
+
+  // Test - 3
+  rocTrueLabels = arma::Row<size_t>("1 0 1 0 1  0 1 0 1 0");
+  rocScoresOfPC = arma::Row<double>("0.8 0.3 0.5 0.4 0.9  0.2 0.7 0.6 0 0.1");
+
+  rocAucScore = 0.24;
+  REQUIRE(ROCAUCScore<0>::Evaluate(rocTrueLabels, rocScoresOfPC)
+          == Approx(rocAucScore).epsilon(1e-7));
+
+  rocAucScore = 0.76;
+  REQUIRE(ROCAUCScore<1>::Evaluate(rocTrueLabels, rocScoresOfPC)
+          == Approx(rocAucScore).epsilon(1e-7));
+
+  // Test - 4 (labels and scores with zero size)
+  rocTrueLabels = arma::Row<size_t>();
+  rocScoresOfPC = arma::Row<double>();
+
+  REQUIRE_THROWS_AS(ROCAUCScore<0>::Evaluate(rocTrueLabels, rocScoresOfPC),
+                    std::invalid_argument);
+  REQUIRE_THROWS_AS(ROCAUCScore<1>::Evaluate(rocTrueLabels, rocScoresOfPC),
+                    std::invalid_argument);
+
+  // Test - 5 (labels and scores with one size)
+  rocTrueLabels = arma::Row<size_t>("1");
+  rocScoresOfPC = arma::Row<double>("0.8");
+
+  REQUIRE_THROWS_AS(ROCAUCScore<0>::Evaluate(rocTrueLabels, rocScoresOfPC),
+                    std::invalid_argument);
+  REQUIRE_THROWS_AS(ROCAUCScore<1>::Evaluate(rocTrueLabels, rocScoresOfPC),
+                    std::invalid_argument);
+
+  // Test - 6 (mismatch labels and scores size)
+  rocTrueLabels = arma::Row<size_t>("1 0 1 0 1  0 1 0 1 0");
+  rocScoresOfPC = arma::Row<double>("0.1 0.2 0.3 0.4 0.5");
+
+  REQUIRE_THROWS_AS(ROCAUCScore<0>::Evaluate(rocTrueLabels, rocScoresOfPC),
+                    std::invalid_argument);
+  REQUIRE_THROWS_AS(ROCAUCScore<1>::Evaluate(rocTrueLabels, rocScoresOfPC),
+                    std::invalid_argument);
+}
+
+/**
+ * Test for confusion matrix.
+ */
+TEST_CASE("ConfusionMatrixTest", "[CVTest]")
+{
+  // Labels that will be considered as "ground truth".
+  arma::Row<size_t> labels("0 0 1 0 0  1 0 1 0 1");
+
+  // Predicted labels.
+  arma::Row<size_t> predictedLabels("0 0 0 0 0  1 1 1 1 1");
+  // Confusion matrix.
+  arma::Mat<int> output;
+  data::ConfusionMatrix(predictedLabels, labels, output, 2);
+  REQUIRE(output(0, 0) == 4);
+  REQUIRE(output(0, 1) == 1);
+  REQUIRE(output(1, 0) == 2);
+  REQUIRE(output(1, 1) == 3);
 }
 
 /**
  * Test metrics for multiclass classification.
  */
-BOOST_AUTO_TEST_CASE(MulticlassClassificationMetricsTest)
+TEST_CASE("MulticlassClassificationMetricsTest", "[CVTest]")
 {
   // Using the same data for training and testing.
   arma::mat data = arma::linspace<arma::rowvec>(1.0, 12.0, 12);
@@ -93,46 +167,47 @@ BOOST_AUTO_TEST_CASE(MulticlassClassificationMetricsTest)
 
   // Assert that the Naive Bayes model really predicts the labels above in
   // response to the data.
-  BOOST_REQUIRE_CLOSE(Accuracy::Evaluate(nb, data, predictedLabels), 1.0, 1e-5);
+  REQUIRE(Accuracy::Evaluate(nb, data, predictedLabels)
+          == Approx(1.0).epsilon(1e-7));
 
   double microaveragedPrecision = double(1 + 1 + 3 + 4) / 12;
-  BOOST_REQUIRE_CLOSE(Precision<Micro>::Evaluate(nb, data, labels),
-      microaveragedPrecision, 1e-5);
+  REQUIRE(Precision<Micro>::Evaluate(nb, data, labels)
+          == Approx(microaveragedPrecision).epsilon(1e-7));
 
   double microaveragedRecall = double(1 + 1 + 3 + 4) / 12;
-  BOOST_REQUIRE_CLOSE(Recall<Micro>::Evaluate(nb, data, labels),
-      microaveragedRecall, 1e-5);
+  REQUIRE(Recall<Micro>::Evaluate(nb, data, labels)
+          == Approx(microaveragedRecall).epsilon(1e-7));
 
   double microaveragedF1 = 2 * microaveragedPrecision * microaveragedRecall /
     (microaveragedPrecision + microaveragedRecall);
-  BOOST_REQUIRE_CLOSE(F1<Micro>::Evaluate(nb, data, labels),
-      microaveragedF1, 1e-5);
+  REQUIRE(F1<Micro>::Evaluate(nb, data, labels)
+          == Approx(microaveragedF1).epsilon(1e-7));
 
   double macroaveragedPrecision = (0.5 + 0.5 + 0.75 + 1.0) / 4;
-  BOOST_REQUIRE_CLOSE(Precision<Macro>::Evaluate(nb, data, labels),
-      macroaveragedPrecision, 1e-5);
+  REQUIRE(Precision<Macro>::Evaluate(nb, data, labels)
+          == Approx(macroaveragedPrecision).epsilon(1e-7));
 
   double macroaveragedRecall = (0.5 + 1.0 / 3 + 1.0 + 1.0) / 4;
-  BOOST_REQUIRE_CLOSE(Recall<Macro>::Evaluate(nb, data, labels),
-      macroaveragedRecall, 1e-5);
+  REQUIRE(Recall<Macro>::Evaluate(nb, data, labels)
+          == Approx(macroaveragedRecall).epsilon(1e-7));
 
   double macroaveragedF1 = (2 * 0.5 * 0.5 / (0.5 + 0.5) +
       2 * 0.5 * (1.0 / 3) / (0.5 + (1.0 / 3)) + 2 * 0.75 * 1.0 / (0.75 + 1.0) +
       2 * 1.0 * 1.0 / (1.0 + 1.0)) / 4;
-  BOOST_REQUIRE_CLOSE(F1<Macro>::Evaluate(nb, data, labels),
-      macroaveragedF1, 1e-5);
+  REQUIRE(F1<Macro>::Evaluate(nb, data, labels)
+          == Approx(macroaveragedF1).epsilon(1e-7));
 }
 
 /**
  * Test the mean squared error.
  */
-BOOST_AUTO_TEST_CASE(MSETest)
+TEST_CASE("MSETest", "[CVTest]")
 {
   // Making two points that define the linear function f(x) = x - 1
   arma::mat trainingData("0 1");
   arma::rowvec trainingResponses("-1 0");
 
-  LinearRegression lr(trainingData, trainingResponses);
+  LinearRegression<> lr(trainingData, trainingResponses);
 
   // Making three responses that differ from the correct ones by 0, 1, and 2
   // respectively
@@ -141,23 +216,67 @@ BOOST_AUTO_TEST_CASE(MSETest)
 
   double expectedMSE = (0 * 0 + 1 * 1 + 2 * 2) / 3.0;
 
-  BOOST_REQUIRE_CLOSE(MSE::Evaluate(lr, data, responses), expectedMSE, 1e-5);
+  REQUIRE(MSE::Evaluate(lr, data, responses)
+          == Approx(expectedMSE).epsilon(1e-7));
+}
+
+/**
+ * Test the R squared metric (R2 Score).
+ */
+TEST_CASE("R2ScoreTest", "[CVTest]")
+{
+  // Making two points that define the linear function f(x) = x - 1.
+  arma::mat trainingData("0 1");
+  arma::rowvec trainingResponses("-1 0");
+
+  LinearRegression<> lr(trainingData, trainingResponses);
+
+  // Making five responses that are the output of regression function f(x)
+  // with some responses having a slight deviation of 0.005.
+  // Mean Responses = (1 + 2 + 3 + 6 + 8)/5 = 4.
+  arma::mat data("2 3 4 7 9");
+  arma::rowvec responses("1 2.005 3 6.005 8.005");
+
+  double expectedR2 = 0.99999779;
+
+  REQUIRE(R2Score<false>::Evaluate(lr, data, responses)
+          == Approx(expectedR2).epsilon(1e-7));
+}
+
+/**
+ * Test the Adjusted R squared metric.
+ */
+TEST_CASE("AdjR2ScoreTest", "[CVTest]")
+{
+  // Making two variables that define the linear function is
+  // f(x1, x2) = x1 + x2.
+  arma::mat X;
+  X = { { 1, 2, 3, 4, 5, 6 },
+        { 2, 3, 4, 5, 6, 7 } };
+  arma::rowvec Y;
+  Y = { 3, 5, 7, 9, 11, 13 };
+
+  LinearRegression<> lr(X, Y);
+
+  // Theoretically Adjusted R squared should be equal 1
+  double expAdjR2 = 1;
+  REQUIRE(std::abs(R2Score<true>::Evaluate(lr, X, Y) - expAdjR2)
+          <= 1e-7);
 }
 
 /**
  * Test the mean squared error with matrix responses.
  */
-BOOST_AUTO_TEST_CASE(MSEMatResponsesTest)
+TEST_CASE("MSEMatResponsesTest", "[CVTest]")
 {
   arma::mat data("1 2");
   arma::mat trainingResponses("1 2; 3 4");
 
-  FFN<MeanSquaredError<>, ConstInitialization> ffn(MeanSquaredError<>(),
+  FFN<MeanSquaredError, ConstInitialization> ffn(MeanSquaredError(),
     ConstInitialization(0));
-  ffn.Add<Linear<>>(1, 2);
-  ffn.Add<IdentityLayer<>>();
+  ffn.Add<Linear>(2);
 
-  RMSProp opt(0.2);
+  ens::RMSProp opt(0.2);
   opt.BatchSize() = 1;
   opt.Shuffle() = false;
   ffn.Train(data, trainingResponses, opt);
@@ -168,7 +287,8 @@ BOOST_AUTO_TEST_CASE(MSEMatResponsesTest)
 
   double expectedMSE = (0 * 0 + 1 * 1 + 2 * 2 + 3 * 3) / 4.0;
 
-  BOOST_REQUIRE_CLOSE(MSE::Evaluate(ffn, data, responses), expectedMSE, 1e-1);
+  REQUIRE(MSE::Evaluate(ffn, data, responses)
+          == Approx(expectedMSE).epsilon(1e-3));
 }
 
 template<typename Class,
@@ -187,13 +307,13 @@ void CheckPredictionsType()
  * Test MetaInfoExtractor correctly recognizes the type of predictions for a
  * given machine learning algorithm.
  */
-BOOST_AUTO_TEST_CASE(PredictionsTypeTest)
+TEST_CASE("PredictionsTypeTest", "[CVTest]")
 {
-  CheckPredictionsType<LinearRegression, arma::rowvec>();
+  CheckPredictionsType<LinearRegression<>, arma::rowvec>();
   // CheckPredictionsType<FFN<>, arma::mat>();
 
   CheckPredictionsType<LogisticRegression<>, arma::Row<size_t>>();
-  CheckPredictionsType<SoftmaxRegression, arma::Row<size_t>>();
+  CheckPredictionsType<SoftmaxRegression<>, arma::Row<size_t>>();
   CheckPredictionsType<HoeffdingTree<>, arma::Row<size_t>, arma::mat>();
   CheckPredictionsType<HoeffdingTree<>, arma::Row<size_t>, arma::imat>();
   CheckPredictionsType<DecisionTree<>, arma::Row<size_t>, arma::mat,
@@ -206,16 +326,16 @@ BOOST_AUTO_TEST_CASE(PredictionsTypeTest)
  * Test MetaInfoExtractor correctly identifies whether a given machine learning
  * algorithm supports weighted learning.
  */
-BOOST_AUTO_TEST_CASE(SupportsWeightsTest)
+TEST_CASE("SupportsWeightsTest", "[CVTest]")
 {
-  static_assert(MetaInfoExtractor<LinearRegression>::SupportsWeights,
+  static_assert(MetaInfoExtractor<LinearRegression<>>::SupportsWeights,
       "Value should be true");
   static_assert(MetaInfoExtractor<DecisionTree<>>::SupportsWeights,
       "Value should be true");
   static_assert(MetaInfoExtractor<DecisionTree<>, arma::mat, arma::urowvec,
       arma::Row<float>>::SupportsWeights, "Value should be true");
 
-  static_assert(!MetaInfoExtractor<LARS>::SupportsWeights,
+  static_assert(!MetaInfoExtractor<LARS<>>::SupportsWeights,
       "Value should be false");
   static_assert(!MetaInfoExtractor<LogisticRegression<>>::SupportsWeights,
       "Value should be false");
@@ -238,9 +358,9 @@ void CheckWeightsType()
  * Test MetaInfoExtractor correctly recognizes the type of weights for a given
  * machine learning algorithm.
  */
-BOOST_AUTO_TEST_CASE(WeightsTypeTest)
+TEST_CASE("WeightsTypeTest", "[CVTest]")
 {
-  CheckWeightsType<LinearRegression, arma::rowvec>();
+  CheckWeightsType<LinearRegression<>, arma::rowvec>();
   CheckWeightsType<DecisionTree<>, arma::rowvec>();
   CheckWeightsType<DecisionTree<>, arma::Row<float>, arma::mat,
       arma::Row<size_t>, arma::Row<float>>();
@@ -250,13 +370,13 @@ BOOST_AUTO_TEST_CASE(WeightsTypeTest)
  * Test MetaInfoExtractor correctly identifies whether a given machine learning
  * algorithm takes a data:DatasetInfo parameter.
  */
-BOOST_AUTO_TEST_CASE(TakesDatasetInfoTest)
+TEST_CASE("TakesDatasetInfoTest", "[CVTest]")
 {
   static_assert(MetaInfoExtractor<DecisionTree<>>::TakesDatasetInfo,
       "Value should be true");
-  static_assert(!MetaInfoExtractor<LinearRegression>::TakesDatasetInfo,
+  static_assert(!MetaInfoExtractor<LinearRegression<>>::TakesDatasetInfo,
       "Value should be false");
-  static_assert(!MetaInfoExtractor<SoftmaxRegression>::TakesDatasetInfo,
+  static_assert(!MetaInfoExtractor<SoftmaxRegression<>>::TakesDatasetInfo,
       "Value should be false");
 }
 
@@ -264,15 +384,15 @@ BOOST_AUTO_TEST_CASE(TakesDatasetInfoTest)
  * Test MetaInfoExtractor correctly identifies whether a given machine learning
  * algorithm takes the numClasses parameter.
  */
-BOOST_AUTO_TEST_CASE(TakesNumClassesTest)
+TEST_CASE("TakesNumClassesTest", "[CVTest]")
 {
   static_assert(MetaInfoExtractor<DecisionTree<>>::TakesNumClasses,
       "Value should be true");
-  static_assert(MetaInfoExtractor<SoftmaxRegression>::TakesNumClasses,
+  static_assert(MetaInfoExtractor<SoftmaxRegression<>>::TakesNumClasses,
       "Value should be true");
-  static_assert(!MetaInfoExtractor<LinearRegression>::TakesNumClasses,
+  static_assert(!MetaInfoExtractor<LinearRegression<>>::TakesNumClasses,
       "Value should be false");
-  static_assert(!MetaInfoExtractor<LARS>::TakesNumClasses,
+  static_assert(!MetaInfoExtractor<LARS<>>::TakesNumClasses,
       "Value should be false");
 }
 
@@ -280,7 +400,7 @@ BOOST_AUTO_TEST_CASE(TakesNumClassesTest)
  * Test the simple cross-validation strategy implementation with the Accuracy
  * metric.
  */
-BOOST_AUTO_TEST_CASE(SimpleCVAccuracyTest)
+TEST_CASE("SimpleCVAccuracyTest", "[CVTest]")
 {
   // Using the first half of data for training and the rest for validation.
   // The validation labels are 75% correct.
@@ -290,13 +410,13 @@ BOOST_AUTO_TEST_CASE(SimpleCVAccuracyTest)
 
   SimpleCV<LogisticRegression<>, Accuracy> cv(0.5, data, labels);
 
-  BOOST_REQUIRE_CLOSE(cv.Evaluate(), 0.75, 1e-5);
+  REQUIRE(cv.Evaluate() == Approx(0.75).epsilon(1e-7));
 }
 
 /**
  * Test the simple cross-validation strategy implementation with the MSE metric.
  */
-BOOST_AUTO_TEST_CASE(SimpleCVMSETest)
+TEST_CASE("SimpleCVMSETest", "[CVTest]")
 {
   // Using the first two points for training and remaining three for validation.
   // See the test MSETest for more explanation.
@@ -305,31 +425,51 @@ BOOST_AUTO_TEST_CASE(SimpleCVMSETest)
 
   double expectedMSE = (0 * 0 + 1 * 1 + 2 * 2) / 3.0;
 
-  SimpleCV<LinearRegression, MSE> cv(0.6, data, responses);
+  SimpleCV<LinearRegression<>, MSE> cv(0.6, data, responses);
 
-  BOOST_REQUIRE_CLOSE(cv.Evaluate(), expectedMSE, 1e-5);
+  REQUIRE(cv.Evaluate() == Approx(expectedMSE).epsilon(1e-7));
 
   arma::mat noiseData("-1 -2 -3 -4 -5");
   arma::rowvec noiseResponses("10 20 30 40 50");
 
-  arma::mat allData = arma::join_rows(noiseData, data);
-  arma::rowvec allResponces = arma::join_rows(noiseResponses, responses);
+  arma::mat allData = join_rows(noiseData, data);
+  arma::rowvec allResponces = join_rows(noiseResponses, responses);
 
-  arma::rowvec weights = arma::join_rows(arma::zeros(noiseData.n_cols).t(),
+  arma::rowvec weights = join_rows(arma::zeros(noiseData.n_cols).t(),
       arma::ones(data.n_cols).t());
 
-  SimpleCV<LinearRegression, MSE> weightedCV(0.3, allData, allResponces,
+  SimpleCV<LinearRegression<>, MSE> weightedCV(0.3, allData, allResponces,
       weights);
 
-  BOOST_REQUIRE_CLOSE(weightedCV.Evaluate(), expectedMSE, 1e-5);
+  REQUIRE(weightedCV.Evaluate() == Approx(expectedMSE).epsilon(1e-7));
 
-  arma::rowvec weights2 = arma::join_rows(arma::zeros(noiseData.n_cols - 1).t(),
+  arma::rowvec weights2 = join_rows(arma::zeros(noiseData.n_cols - 1).t(),
       arma::ones(data.n_cols + 1).t());
 
-  SimpleCV<LinearRegression, MSE> weightedCV2(0.3, allData, allResponces,
+  SimpleCV<LinearRegression<>, MSE> weightedCV2(0.3, allData, allResponces,
       weights2);
 
-  BOOST_REQUIRE_GT(std::abs(weightedCV2.Evaluate() - expectedMSE), 1e-5);
+  REQUIRE(std::abs(weightedCV2.Evaluate() - expectedMSE) > 1e-5);
+}
+
+/**
+ * Test that scores of -nan are filtered out.
+ */
+TEST_CASE("FilterNANCVTest", "[CVTest]")
+{
+  // Create a dataset with only one positive label, so it will not be in every
+  // fold.
+  arma::mat data(3, 10, arma::fill::randu);
+  arma::Row<size_t> labels(10, arma::fill::zeros);
+  labels[0] = 1;
+
+  const size_t numClasses = 2;
+  KFoldCV<NaiveBayesClassifier<>, F1<Binary>> kfoldcv(2, data, labels,
+      numClasses);
+
+  const double result = kfoldcv.Evaluate();
+  REQUIRE(!std::isnan(result));
+  REQUIRE(!std::isinf(result));
 }
 
 template<typename... DTArgs>
@@ -346,7 +486,7 @@ arma::Row<size_t> PredictLabelsWithDT(const arma::mat& data,
  * Test the simple cross-validation strategy implementation with decision trees
  * constructed in multiple ways.
  */
-BOOST_AUTO_TEST_CASE(SimpleCVWithDTTest)
+TEST_CASE("SimpleCVWithDTTest", "[CVTest]")
 {
   arma::mat data;
   arma::Row<size_t> labels;
@@ -366,60 +506,60 @@ BOOST_AUTO_TEST_CASE(SimpleCVWithDTTest)
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(testData,
         trainingData, trainingLabels, numClasses, minimumLeafSize);
     SimpleCV<DecisionTree<InformationGain>, Accuracy> cv(0.5, data,
-        arma::join_rows(trainingLabels, predictedLabels), numClasses);
-    BOOST_REQUIRE_CLOSE(cv.Evaluate(minimumLeafSize), 1.0, 1e-5);
+        join_rows(trainingLabels, predictedLabels), numClasses);
+    REQUIRE(cv.Evaluate(minimumLeafSize) == Approx(1.0).epsilon(1e-7));
   }
   {
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(testData,
         trainingData, datasetInfo, trainingLabels, numClasses, minimumLeafSize);
     SimpleCV<DecisionTree<InformationGain>, Accuracy> cv(0.5, data, datasetInfo,
-        arma::join_rows(trainingLabels, predictedLabels), numClasses);
-    BOOST_REQUIRE_CLOSE(cv.Evaluate(minimumLeafSize), 1.0, 1e-5);
+        join_rows(trainingLabels, predictedLabels), numClasses);
+    REQUIRE(cv.Evaluate(minimumLeafSize) == Approx(1.0).epsilon(1e-7));
   }
   {
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(testData,
         trainingData, trainingLabels, numClasses, weights, minimumLeafSize);
     SimpleCV<DecisionTree<InformationGain>, Accuracy> cv(0.5, data,
-        arma::join_rows(trainingLabels, predictedLabels), numClasses, weights);
-    BOOST_REQUIRE_CLOSE(cv.Evaluate(minimumLeafSize), 1.0, 1e-5);
+        join_rows(trainingLabels, predictedLabels), numClasses, weights);
+    REQUIRE(cv.Evaluate(minimumLeafSize) == Approx(1.0).epsilon(1e-7));
   }
   {
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(testData,
         trainingData, datasetInfo, trainingLabels, numClasses, weights,
         minimumLeafSize);
     SimpleCV<DecisionTree<InformationGain>, Accuracy> cv(0.5, data, datasetInfo,
-        arma::join_rows(trainingLabels, predictedLabels), numClasses, weights);
-    BOOST_REQUIRE_CLOSE(cv.Evaluate(minimumLeafSize), 1.0, 1e-5);
+        join_rows(trainingLabels, predictedLabels), numClasses, weights);
+    REQUIRE(cv.Evaluate(minimumLeafSize) == Approx(1.0).epsilon(1e-7));
   }
 }
 
 /**
  * Test k-fold cross-validation with the MSE metric.
  */
-BOOST_AUTO_TEST_CASE(KFoldCVMSETest)
+TEST_CASE("KFoldCVMSETest", "[CVTest]")
 {
   // Defining dataset with two sets of responses for the same two data points.
   arma::mat data("0 1  0 1");
   arma::rowvec responses("0 1  1 3");
 
   // 2-fold cross-validation, no shuffling.
-  KFoldCV<LinearRegression, MSE> cv(2, data, responses, false);
+  KFoldCV<LinearRegression<>, MSE> cv(2, data, responses, false);
 
   // In each of two validation tests the MSE value should be the same.
   double expectedMSE =
       double((1 - 0) * (1 - 0) + (3 - 1) * (3 - 1)) / 2 * 2 / 2;
 
-  BOOST_REQUIRE_CLOSE(cv.Evaluate(), expectedMSE, 1e-5);
+  REQUIRE(cv.Evaluate() == Approx(expectedMSE).epsilon(1e-7));
 
   // Assert we can access a trained model without the exception of
   // uninitialization.
-  cv.Model();
+  REQUIRE_NOTHROW(cv.Model());
 }
 
 /**
  * Test k-fold cross-validation with the Accuracy metric.
  */
-BOOST_AUTO_TEST_CASE(KFoldCVAccuracyTest)
+TEST_CASE("KFoldCVAccuracyTest", "[CVTest]")
 {
   // Making a 10-points dataset. The last point should be classified wrong when
   // it is tested separately.
@@ -435,25 +575,53 @@ BOOST_AUTO_TEST_CASE(KFoldCVAccuracyTest)
   // fail with the remaining one.
   double expectedAccuracy = (9 * 1.0 + 0.0) / 10;
 
-  BOOST_REQUIRE_CLOSE(cv.Evaluate(), expectedAccuracy, 1e-5);
+  REQUIRE(cv.Evaluate() == Approx(expectedAccuracy).epsilon(1e-7));
 
   // Assert we can access a trained model without the exception of
   // uninitialization.
-  cv.Model();
+  REQUIRE_NOTHROW(cv.Model());
+}
+
+/**
+ * Test k-fold cross-validation with the perceptron.
+ */
+TEST_CASE("KFoldCVPerceptronTest", "[CVTest]")
+{
+  // Basically the same as the test above (for Naive Bayes), but with the
+  // perceptron.
+
+  // Making a 10-points dataset. All points should always be correctly
+  // classified.
+  arma::mat data("0 0 0 0 0 1 1 1 1 1");
+  arma::Row<size_t> labels("0 0 0 0 0 1 1 1 1 1");
+  size_t numClasses = 2;
+
+  // 10-fold cross-validation, no shuffling.
+  KFoldCV<Perceptron<>, Accuracy> cv(10, data, labels, numClasses);
+
+  // We should succeed in classifying separately the first nine samples, and
+  // fail with the remaining one.
+  double expectedAccuracy = 1.0;
+
+  REQUIRE(cv.Evaluate() == Approx(expectedAccuracy).epsilon(1e-7));
+
+  // Assert we can access a trained model without the exception of
+  // uninitialization.
+  REQUIRE_NOTHROW(cv.Model());
 }
 
 /**
  * Test k-fold cross-validation with weighted linear regression.
  */
-BOOST_AUTO_TEST_CASE(KFoldCVWithWeightedLRTest)
+TEST_CASE("KFoldCVWithWeightedLRTest", "[CVTest]")
 {
   // Each fold will be filled with this dataset.
   arma::mat data("1 2 3 4");
   arma::rowvec responses("1 2 30 40");
   arma::rowvec weights("1 1 0 0");
 
-  KFoldCV<LinearRegression, MSE> cv(2, arma::join_rows(data, data),
-      arma::join_rows(responses, responses), arma::join_rows(weights, weights),
+  KFoldCV<LinearRegression<>, MSE> cv(2, join_rows(data, data),
+      join_rows(responses, responses), join_rows(weights, weights),
       false);
   cv.Evaluate();
 
@@ -462,14 +630,14 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithWeightedLRTest)
 
   double mse = MSE::Evaluate(cv.Model(), testData, testResponses);
 
-  BOOST_REQUIRE_CLOSE(1.0 - mse, 1.0, 1e-5);
+  REQUIRE((1.0 - mse) == Approx(1.0).epsilon(1e-7));
 }
 
 /**
  * Test k-fold cross-validation with decision trees constructed in multiple
  * ways.
  */
-BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
+TEST_CASE("KFoldCVWithDTTest", "[CVTest]")
 {
   arma::mat originalData;
   arma::Row<size_t> originalLabels;
@@ -481,9 +649,9 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
   arma::Row<size_t> labels = originalLabels.cols(0, 1199);
   arma::rowvec weights(data.n_cols, arma::fill::randu);
 
-  arma::mat doubledData = arma::join_rows(data, data);
-  arma::Row<size_t> doubledLabels = arma::join_rows(labels, labels);
-  arma::rowvec doubledWeights = arma::join_rows(weights, weights);
+  arma::mat doubledData = join_rows(data, data);
+  arma::Row<size_t> doubledLabels = join_rows(labels, labels);
+  arma::rowvec doubledWeights = join_rows(weights, weights);
 
   size_t numClasses = 5;
   size_t minimumLeafSize = 8;
@@ -495,7 +663,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(data, data, labels,
         numClasses, minimumLeafSize);
     double accuracy = Accuracy::Evaluate(cv.Model(), data, predictedLabels);
-    BOOST_REQUIRE_CLOSE(accuracy, 1.0, 1e-5);
+    REQUIRE(accuracy == Approx(1.0).epsilon(1e-7));
   }
   {
     KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(2, doubledData,
@@ -504,7 +672,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(data, data,
         datasetInfo, labels, numClasses, minimumLeafSize);
     double accuracy = Accuracy::Evaluate(cv.Model(), data, predictedLabels);
-    BOOST_REQUIRE_CLOSE(accuracy, 1.0, 1e-5);
+    REQUIRE(accuracy == Approx(1.0).epsilon(1e-7));
   }
   {
     KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(2, doubledData,
@@ -513,7 +681,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(data, data, labels,
         numClasses, weights, minimumLeafSize);
     double accuracy = Accuracy::Evaluate(cv.Model(), data, predictedLabels);
-    BOOST_REQUIRE_CLOSE(accuracy, 1.0, 1e-5);
+    REQUIRE(accuracy == Approx(1.0).epsilon(1e-7));
   }
   {
     KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(2, doubledData,
@@ -522,7 +690,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(data, data,
         datasetInfo, labels, numClasses, weights, minimumLeafSize);
     double accuracy = Accuracy::Evaluate(cv.Model(), data, predictedLabels);
-    BOOST_REQUIRE_CLOSE(accuracy, 1.0, 1e-5);
+    REQUIRE(accuracy == Approx(1.0).epsilon(1e-7));
   }
 }
 
@@ -530,7 +698,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
  * Test k-fold cross-validation with decision trees constructed in multiple
  * ways, but with larger k and no shuffling.
  */
-BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeKNoShuffle)
+TEST_CASE("KFoldCVWithDTTestLargeKNoShuffle", "[CVTest]")
 {
   arma::mat data;
   arma::Row<size_t> labels;
@@ -547,7 +715,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeKNoShuffle)
 
   // This is a very loose tolerance, but we expect about the same as we would
   // from an individual decision tree training.
-  BOOST_REQUIRE_GT(accuracy, 0.7);
+  REQUIRE(accuracy > 0.7);
 }
 
 /**
@@ -556,7 +724,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeKNoShuffle)
  * cross-validation bin is not even (the last is smaller), and also with no
  * shuffling.
  */
-BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestUnevenBinsNoShuffle)
+TEST_CASE("KFoldCVWithDTTestUnevenBinsNoShuffle", "[CVTest]")
 {
   arma::mat data;
   arma::Row<size_t> labels;
@@ -573,14 +741,14 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestUnevenBinsNoShuffle)
 
   // This is a very loose tolerance, but we expect about the same as we would
   // from an individual decision tree training.
-  BOOST_REQUIRE_GT(accuracy, 0.7);
+  REQUIRE(accuracy > 0.7);
 }
 
 /**
  * Test k-fold cross-validation with decision trees constructed in multiple
  * ways, but with larger k.
  */
-BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeK)
+TEST_CASE("KFoldCVWithDTTestLargeK", "[CVTest]")
 {
   arma::mat data;
   arma::Row<size_t> labels;
@@ -597,7 +765,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeK)
 
   // This is a very loose tolerance, but we expect about the same as we would
   // from an individual decision tree training.
-  BOOST_REQUIRE_GT(accuracy, 0.7);
+  REQUIRE(accuracy > 0.7);
 }
 
 /**
@@ -605,7 +773,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeK)
  * ways, but with larger k such that the number of points in each
  * cross-validation bin is not even (the last is smaller).
  */
-BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestUnevenBins)
+TEST_CASE("KFoldCVWithDTTestUnevenBins", "[CVTest]")
 {
   arma::mat data;
   arma::Row<size_t> labels;
@@ -622,14 +790,14 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestUnevenBins)
 
   // This is a very loose tolerance, but we expect about the same as we would
   // from an individual decision tree training.
-  BOOST_REQUIRE_GT(accuracy, 0.7);
+  REQUIRE(accuracy > 0.7);
 }
 
 /**
  * Test k-fold cross-validation with decision trees constructed in multiple
  * ways, but with larger k and weights.
  */
-BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeKWeighted)
+TEST_CASE("KFoldCVWithDTTestLargeKWeighted", "[CVTest]")
 {
   arma::mat data;
   arma::Row<size_t> labels;
@@ -647,7 +815,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeKWeighted)
 
   // This is a very loose tolerance, but we expect about the same as we would
   // from an individual decision tree training.
-  BOOST_REQUIRE_GT(accuracy, 0.7);
+  REQUIRE(accuracy > 0.7);
 }
 
 /**
@@ -655,7 +823,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeKWeighted)
  * ways, but with larger k such that the number of points in each
  * cross-validation bin is not even (the last is smaller) and weights.
  */
-BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestUnevenBinsWeighted)
+TEST_CASE("KFoldCVWithDTTestUnevenBinsWeighted", "[CVTest]")
 {
   arma::mat data;
   arma::Row<size_t> labels;
@@ -673,7 +841,20 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestUnevenBinsWeighted)
 
   // This is a very loose tolerance, but we expect about the same as we would
   // from an individual decision tree training.
-  BOOST_REQUIRE_GT(accuracy, 0.7);
+  REQUIRE(accuracy > 0.7);
 }
 
-BOOST_AUTO_TEST_SUITE_END();
+/**
+ * Test Silhouette Score
+ */
+TEST_CASE("SilhouetteScoreTest", "[CVTest]")
+{
+  arma::mat X;
+  X = { { 0, 1, 1, 0, 0 },
+        { 0, 1, 2, 0, 0 },
+        { 1, 1, 3, 2, 0 } };
+  arma::Row<size_t> labels = { 0, 1, 2, 0, 0 };
+  EuclideanDistance metric;
+  double silhouetteScore = SilhouetteScore::Overall(X, labels, metric);
+  REQUIRE(silhouetteScore == Approx(0.1121684822489150).epsilon(1e-7));
+}
