@@ -1,5 +1,5 @@
 /**
- * @file dbscan_test.cpp
+ * @file tests/dbscan_test.cpp
  * @author Ryan Curtin
  *
  * Test the DBSCAN implementation.
@@ -10,18 +10,26 @@
  * http://www.opensource.org/licenses/BSD-3-Clause for more information.
  */
 #include <mlpack/core.hpp>
-#include <mlpack/methods/dbscan/dbscan.hpp>
+#include <mlpack/methods/dbscan.hpp>
 
-#include <boost/test/unit_test.hpp>
-#include "test_tools.hpp"
+#include "test_catch_tools.hpp"
+#include "catch.hpp"
 
 using namespace mlpack;
-using namespace mlpack::dbscan;
-using namespace mlpack::distribution;
 
-BOOST_AUTO_TEST_SUITE(DBSCANTest);
+/**
+ * A couple of handful declarations for float32 testing.
+ * These will be removed when we refactor the Bounds to accept MatType.
+ * For now, we will keep the following declarations.
+ */
+template<typename MetricType>
+using FloatHRectBound = HRectBound<MetricType, float>;
 
-BOOST_AUTO_TEST_CASE(OneClusterTest)
+template<typename MetricType, typename StatisticType, typename MatType>
+using FloatKDTree = BinarySpaceTree<MetricType, StatisticType, MatType,
+                                    FloatHRectBound, MidpointSplit>;
+
+TEST_CASE("OneClusterTest", "[DBSCANTest]")
 {
   // Make sure that if we have points in the unit box, and if we set epsilon
   // large enough, all points end up as in one cluster.
@@ -32,16 +40,16 @@ BOOST_AUTO_TEST_CASE(OneClusterTest)
   arma::Row<size_t> assignments;
   const size_t clusters = d.Cluster(points, assignments);
 
-  BOOST_REQUIRE_EQUAL(clusters, 1);
-  BOOST_REQUIRE_EQUAL(assignments.n_elem, points.n_cols);
+  REQUIRE(clusters == 1);
+  REQUIRE(assignments.n_elem == points.n_cols);
   for (size_t i = 0; i < assignments.n_elem; ++i)
-    BOOST_REQUIRE_EQUAL(assignments[i], 0);
+    REQUIRE(assignments[i] == 0);
 }
 
 /**
  * When epsilon is small enough, every point returned should be noise.
  */
-BOOST_AUTO_TEST_CASE(TinyEpsilonTest)
+TEST_CASE("TinyEpsilonTest", "[DBSCANTest]")
 {
   arma::mat points(10, 200, arma::fill::randu);
 
@@ -50,16 +58,16 @@ BOOST_AUTO_TEST_CASE(TinyEpsilonTest)
   arma::Row<size_t> assignments;
   const size_t clusters = d.Cluster(points, assignments);
 
-  BOOST_REQUIRE_EQUAL(clusters, 0);
-  BOOST_REQUIRE_EQUAL(assignments.n_elem, points.n_cols);
+  REQUIRE(clusters == 0);
+  REQUIRE(assignments.n_elem == points.n_cols);
   for (size_t i = 0; i < assignments.n_elem; ++i)
-    BOOST_REQUIRE_EQUAL(assignments[i], SIZE_MAX);
+    REQUIRE(assignments[i] == SIZE_MAX);
 }
 
 /**
  * Check that outliers are properly labeled as noise.
  */
-BOOST_AUTO_TEST_CASE(OutlierTest)
+TEST_CASE("OutlierTest", "[DBSCANTest]")
 {
   arma::mat points(2, 200, arma::fill::randu);
 
@@ -73,17 +81,17 @@ BOOST_AUTO_TEST_CASE(OutlierTest)
   arma::Row<size_t> assignments;
   const size_t clusters = d.Cluster(points, assignments);
 
-  BOOST_REQUIRE_GT(clusters, 0);
-  BOOST_REQUIRE_EQUAL(assignments.n_elem, points.n_cols);
-  BOOST_REQUIRE_EQUAL(assignments[15], SIZE_MAX);
-  BOOST_REQUIRE_EQUAL(assignments[45], SIZE_MAX);
-  BOOST_REQUIRE_EQUAL(assignments[101], SIZE_MAX);
+  REQUIRE(clusters > 0);
+  REQUIRE(assignments.n_elem == points.n_cols);
+  REQUIRE(assignments[15] == SIZE_MAX);
+  REQUIRE(assignments[45] == SIZE_MAX);
+  REQUIRE(assignments[101] == SIZE_MAX);
 }
 
 /**
  * Check that the Gaussian clusters are correctly found.
  */
-BOOST_AUTO_TEST_CASE(GaussiansTest)
+TEST_CASE("GaussiansTest", "[DBSCANTest]")
 {
   arma::mat points(3, 300);
 
@@ -103,7 +111,7 @@ BOOST_AUTO_TEST_CASE(GaussiansTest)
   arma::Row<size_t> assignments;
   arma::mat centroids;
   const size_t clusters = d.Cluster(points, assignments, centroids);
-  BOOST_REQUIRE_EQUAL(clusters, 3);
+  REQUIRE(clusters == 3);
 
   // Our centroids should be close to one of our Gaussians.
   arma::Row<size_t> matches(3);
@@ -118,35 +126,35 @@ BOOST_AUTO_TEST_CASE(GaussiansTest)
       matches(2) = j;
   }
 
-  BOOST_REQUIRE_NE(matches(0), matches(1));
-  BOOST_REQUIRE_NE(matches(1), matches(2));
-  BOOST_REQUIRE_NE(matches(2), matches(0));
+  REQUIRE(matches(0) != matches(1));
+  REQUIRE(matches(1) != matches(2));
+  REQUIRE(matches(2) != matches(0));
 
-  BOOST_REQUIRE_NE(matches(0), 3);
-  BOOST_REQUIRE_NE(matches(1), 3);
-  BOOST_REQUIRE_NE(matches(2), 3);
+  REQUIRE(matches(0) != 3);
+  REQUIRE(matches(1) != 3);
+  REQUIRE(matches(2) != 3);
 
   for (size_t i = 0; i < 100; ++i)
   {
     // Each point should either be noise or in cluster matches(0).
-    BOOST_REQUIRE_NE(assignments(i), matches(1));
-    BOOST_REQUIRE_NE(assignments(i), matches(2));
+    REQUIRE(assignments(i) != matches(1));
+    REQUIRE(assignments(i) != matches(2));
   }
 
   for (size_t i = 100; i < 200; ++i)
   {
-    BOOST_REQUIRE_NE(assignments(i), matches(0));
-    BOOST_REQUIRE_NE(assignments(i), matches(2));
+    REQUIRE(assignments(i) != matches(0));
+    REQUIRE(assignments(i) != matches(2));
   }
 
   for (size_t i = 200; i < 300; ++i)
   {
-    BOOST_REQUIRE_NE(assignments(i), matches(0));
-    BOOST_REQUIRE_NE(assignments(i), matches(1));
+    REQUIRE(assignments(i) != matches(0));
+    REQUIRE(assignments(i) != matches(1));
   }
 }
 
-BOOST_AUTO_TEST_CASE(OneClusterSingleModeTest)
+TEST_CASE("OneClusterSingleModeTest", "[DBSCANTest]")
 {
   // Make sure that if we have points in the unit box, and if we set epsilon
   // large enough, all points end up as in one cluster.
@@ -157,16 +165,16 @@ BOOST_AUTO_TEST_CASE(OneClusterSingleModeTest)
   arma::Row<size_t> assignments;
   const size_t clusters = d.Cluster(points, assignments);
 
-  BOOST_REQUIRE_EQUAL(clusters, 1);
-  BOOST_REQUIRE_EQUAL(assignments.n_elem, points.n_cols);
+  REQUIRE(clusters == 1);
+  REQUIRE(assignments.n_elem == points.n_cols);
   for (size_t i = 0; i < assignments.n_elem; ++i)
-    BOOST_REQUIRE_EQUAL(assignments[i], 0);
+    REQUIRE(assignments[i] == 0);
 }
 
 /**
  * When epsilon is small enough, every point returned should be noise.
  */
-BOOST_AUTO_TEST_CASE(TinyEpsilonSingleModeTest)
+TEST_CASE("TinyEpsilonSingleModeTest", "[DBSCANTest]")
 {
   arma::mat points(10, 200, arma::fill::randu);
 
@@ -175,16 +183,16 @@ BOOST_AUTO_TEST_CASE(TinyEpsilonSingleModeTest)
   arma::Row<size_t> assignments;
   const size_t clusters = d.Cluster(points, assignments);
 
-  BOOST_REQUIRE_EQUAL(clusters, 0);
-  BOOST_REQUIRE_EQUAL(assignments.n_elem, points.n_cols);
+  REQUIRE(clusters == 0);
+  REQUIRE(assignments.n_elem == points.n_cols);
   for (size_t i = 0; i < assignments.n_elem; ++i)
-    BOOST_REQUIRE_EQUAL(assignments[i], SIZE_MAX);
+    REQUIRE(assignments[i] == SIZE_MAX);
 }
 
 /**
  * Check that outliers are properly labeled as noise.
  */
-BOOST_AUTO_TEST_CASE(OutlierSingleModeTest)
+TEST_CASE("OutlierSingleModeTest", "[DBSCANTest]")
 {
   arma::mat points(2, 200, arma::fill::randu);
 
@@ -198,17 +206,46 @@ BOOST_AUTO_TEST_CASE(OutlierSingleModeTest)
   arma::Row<size_t> assignments;
   const size_t clusters = d.Cluster(points, assignments);
 
-  BOOST_REQUIRE_GT(clusters, 0);
-  BOOST_REQUIRE_EQUAL(assignments.n_elem, points.n_cols);
-  BOOST_REQUIRE_EQUAL(assignments[15], SIZE_MAX);
-  BOOST_REQUIRE_EQUAL(assignments[45], SIZE_MAX);
-  BOOST_REQUIRE_EQUAL(assignments[101], SIZE_MAX);
+  REQUIRE(clusters > 0);
+  REQUIRE(assignments.n_elem == points.n_cols);
+  REQUIRE(assignments[15] == SIZE_MAX);
+  REQUIRE(assignments[45] == SIZE_MAX);
+  REQUIRE(assignments[101] == SIZE_MAX);
+}
+
+/**
+ * Check that outliers are properly labeled as noise
+ * in the case of float32.
+ */
+TEST_CASE("Float32OutlierSingleModeTest", "[DBSCANTest]")
+{
+  arma::Mat<float> points(2, 200, arma::fill::randu);
+
+  // Add 3 outliers.
+  points.col(15) = arma::Col<float>("10.3 1.6");
+  points.col(45) = arma::Col<float>("-100 0.0");
+  points.col(101) = arma::Col<float>("1.5 1.5");
+
+  DBSCAN<RangeSearch<
+         EuclideanDistance,
+         arma::Mat<float>,
+         FloatKDTree>,
+         OrderedPointSelection> d(0.1, 3, false);
+
+  arma::Row<size_t> assignments;
+  const size_t clusters = d.Cluster(points, assignments);
+
+  REQUIRE(clusters > 0);
+  REQUIRE(assignments.n_elem == points.n_cols);
+  REQUIRE(assignments[15] == SIZE_MAX);
+  REQUIRE(assignments[45] == SIZE_MAX);
+  REQUIRE(assignments[101] == SIZE_MAX);
 }
 
 /**
  * Check that the Gaussian clusters are correctly found.
  */
-BOOST_AUTO_TEST_CASE(GaussiansSingleModeTest)
+TEST_CASE("GaussiansSingleModeTest", "[DBSCANTest]")
 {
   arma::mat points(3, 300);
 
@@ -228,7 +265,7 @@ BOOST_AUTO_TEST_CASE(GaussiansSingleModeTest)
   arma::Row<size_t> assignments;
   arma::mat centroids;
   const size_t clusters = d.Cluster(points, assignments, centroids);
-  BOOST_REQUIRE_EQUAL(clusters, 3);
+  REQUIRE(clusters == 3);
 
   // Our centroids should be close to one of our Gaussians.
   arma::Row<size_t> matches(3);
@@ -243,32 +280,99 @@ BOOST_AUTO_TEST_CASE(GaussiansSingleModeTest)
       matches(2) = j;
   }
 
-  BOOST_REQUIRE_NE(matches(0), matches(1));
-  BOOST_REQUIRE_NE(matches(1), matches(2));
-  BOOST_REQUIRE_NE(matches(2), matches(0));
+  REQUIRE(matches(0) != matches(1));
+  REQUIRE(matches(1) != matches(2));
+  REQUIRE(matches(2) != matches(0));
 
-  BOOST_REQUIRE_NE(matches(0), 3);
-  BOOST_REQUIRE_NE(matches(1), 3);
-  BOOST_REQUIRE_NE(matches(2), 3);
+  REQUIRE(matches(0) != 3);
+  REQUIRE(matches(1) != 3);
+  REQUIRE(matches(2) != 3);
 
   for (size_t i = 0; i < 100; ++i)
   {
     // Each point should either be noise or in cluster matches(0).
-    BOOST_REQUIRE_NE(assignments(i), matches(1));
-    BOOST_REQUIRE_NE(assignments(i), matches(2));
+    REQUIRE(assignments(i) != matches(1));
+    REQUIRE(assignments(i) != matches(2));
   }
 
   for (size_t i = 100; i < 200; ++i)
   {
-    BOOST_REQUIRE_NE(assignments(i), matches(0));
-    BOOST_REQUIRE_NE(assignments(i), matches(2));
+    REQUIRE(assignments(i) != matches(0));
+    REQUIRE(assignments(i) != matches(2));
   }
 
   for (size_t i = 200; i < 300; ++i)
   {
-    BOOST_REQUIRE_NE(assignments(i), matches(0));
-    BOOST_REQUIRE_NE(assignments(i), matches(1));
+    REQUIRE(assignments(i) != matches(0));
+    REQUIRE(assignments(i) != matches(1));
   }
 }
 
-BOOST_AUTO_TEST_SUITE_END();
+/**
+ * Check that OrderedPointSelection works correctly.
+ */
+TEST_CASE("OrderedPointSelectionTest", "[DBSCANTest]")
+{
+  arma::mat points(10, 200, arma::fill::randu);
+
+  DBSCAN<> d(2.0, 2);
+
+  arma::Row<size_t> assignments;
+  const size_t clusters = d.Cluster(points, assignments);
+
+  REQUIRE(clusters == 1);
+
+  // The number of assignments returned should be the same as points.
+  REQUIRE(assignments.n_elem == points.n_cols);
+}
+
+/**
+ * Check that RandomPointSelection works correctly.
+ */
+TEST_CASE("RandomPointSelectionTest", "[DBSCANTest]")
+{
+  arma::mat points(10, 200, arma::fill::randu);
+
+  DBSCAN<RangeSearch<>, RandomPointSelection> d(2.0, 2);
+
+  arma::Row<size_t> assignments;
+  const size_t clusters = d.Cluster(points, assignments);
+
+  REQUIRE(clusters == 1);
+
+  // The number of assignments returned should be the same as points.
+  REQUIRE(assignments.n_elem == points.n_cols);
+}
+
+/**
+ * Check that noise points do not accidentally connect clusters.
+ * See issue #3339.  (Thanks @iad-ABDUL-RAOUF!)
+ */
+TEST_CASE("NoiseConnectionTest", "[DBSCANTest]")
+{
+  arma::mat dataset({
+      // cluster 1           cluster 2            noise
+      { 0.0, 0.5,  0.5, 1.0, 3.0, 3.5,  3.5, 4.0, 2.0 },
+      { 0.0, 0.5, -0.5, 0.0, 0.0, 0.5, -0.5, 0.0, 0.0 }});
+
+  // Now perform clustering.
+  const double epsilon = 1.1;
+  size_t minPts = 4;
+
+  DBSCAN<> dbscan(epsilon, minPts, false);
+
+  arma::Row<size_t> labels;
+  arma::mat centroids;
+
+  size_t numClusters = dbscan.Cluster(dataset, labels, centroids);
+
+  // The noisy element should not link the two clusters together, since it has
+  // less than minPts neighbors.
+  REQUIRE(numClusters == 2);
+
+  // Now make sure the same is true with batch clustering.
+  dbscan = DBSCAN<>(epsilon, minPts, true);
+  numClusters = dbscan.Cluster(dataset, labels, centroids);
+
+  REQUIRE(numClusters == 2);
+}
